#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

struct Hash {
	static const int MA = 1e9 + 7, MB = 1e9 + 9;
	int a, b;
	Hash() : a(0), b(0) {}
	Hash(int a, int b) : a(a), b(b) {}
	Hash(int a) : a(a), b(a) {}
	Hash operator +(const Hash &other) const {
		return Hash((a + other.a) % MA, (b + other.b) % MB);
	}
	Hash operator -(const Hash &other) const {
	 	return Hash((a - other.a + MA) % MA, (b - other.b + MB) % MB);
	}
	Hash operator *(const Hash &other) const {
		return Hash(((ll)a * other.a) % MA, ((ll)b * other.b) % MB);
	}
	bool operator ==(const Hash &other) const {
		return a == other.a && b == other.b;
	}
	bool operator !=(const Hash &other) const {
	 	return a != other.a || b != other.b;
	}
};

const Hash P = 239;

struct Fenwick {
	int n;
	vector <int> f;
	Fenwick(int nn) {
		n = nn;
		f.resize(n);
 	}
 	void modify(int id, int val) {
 	 	for (int i = id; i < n; i |= i + 1) {
 	 		f[i] += val;
 	 	}
 	}
 	int pref(int r) {
 		int res = 0;
 		for (int i = r; i >= 0; i &= i + 1, --i) {
 			res += f[i];
 		}
 		return res;
 	}
 	int get(int l, int r) {
 		return pref(r) - pref(l - 1);
 	}
 	void clear() {
 		for (int i = 0; i < n; ++i) f[i] = 0;
 	}
};

ll solve(string s) {
	int n = s.size();
	vector <Hash> deg(n + 1);
	deg[0] = 1;
	vector <Hash> h(n + 1), rh(n + 1);
	for (int i = 0; i < n; ++i) {
		deg[i + 1] = deg[i] * P;
	}
	for (int i = 0; i < n; ++i) {
		h[i + 1] = h[i] * P + s[i];
	}
	for (int i = n - 1; i >= 0; --i) {
		rh[i] = rh[i + 1] * P + s[i];
	}
	auto get1 = [&] (int l, int len) {
		return h[l + len] - h[l] * deg[len];
	};
	auto get2 = [&] (int l, int len) {
		return rh[l] - rh[l + len] * deg[len];		
	};
	auto is_pal = [&] (int l, int r) {
		int len = r - l + 1;
		return get1(l, len) == get2(l, len);
	};
	ll ans = 0;
	{
		vector <int> len(n);
		for (int i = 0; i < n; ++i) {
			int l = 1, r = min(i + 2, n - i + 1);
			while (r - l > 1) {
			 	int m = (l + r) >> 1;
			 	if (is_pal(i - m + 1, i + m - 1)) l = m;
			 	else r = m;
		 	}
		 	len[i] = l;
		}
		for (int i = 0; i < n; ++i) --len[i];
		vector <pair <int, int>> have;
		for (int i = 0; i < n; ++i) {
			have.emplace_back(i - 1 - 2 * len[i], i);
		}
		sort(have.begin(), have.end());
		int uk = 0;
		vector <Fenwick> a(2, Fenwick(n));
		for (int i = 0; i < n; ++i) {
		 	while (uk < (int)have.size() && i >= have[uk].first) {
		 		int id = have[uk].second;
		 		a[id % 2].modify(id, 1);
		 		++uk;
		 	}
		 	int l = i, r = i + 2 * len[i] + 1;
		 	l = max(l, 0);
		 	r = min(r, n - 1);
		 	if (l <= r) {
		 	 	ans += a[(i % 2) ^ 1].get(l, r);
		 	}
		}
	}
	{
		vector <int> len(n);
		for (int i = 0; i < n; ++i) {
		 	int l = 0, r = min(i + 2, n - (i + 1) + 1);
		 	while (r - l > 1) {
		 		int m = (l + r) >> 1;
		 		if (is_pal(i - m + 1, i + 1 + m - 1)) l = m;
		 		else r = m;
		 	}
		 	len[i] = l;
		}
		vector <pair <int, int>> have;
		for (int i = 0; i < n; ++i) {
			have.emplace_back(i - 2 * len[i], i);
		}
		sort(have.begin(), have.end());
		int uk = 0;
		vector <Fenwick> a(2, Fenwick(n));
		for (int i = 0; i < n; ++i) {
		 	while (uk < (int)have.size() && i >= have[uk].first) {
		 		int id = have[uk].second;
		 		a[id % 2].modify(id, 1);
		 		++uk;
		 	}
		 	int l = i + 1, r = 2 * len[i] + i;
		 	l = max(l, 0);
		 	r = min(r, n - 1);
		 	if (l <= r) ans += a[i % 2].get(l, r);
		}
	}
	return ans;
}	

int main() {
	ios_base::sync_with_stdio(false); cin.tie(0);
	#ifdef LOCAL
	freopen("input.txt", "r", stdin);
	#endif
	int n;
	cin >> n;
	string s;
	cin >> s;
	cout << solve(s) << '\n';
}