#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <cmath>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <ctime>

using namespace std;

typedef long long ll;
typedef long double ld;

const int N = (1 << 19);

void IOOptimize() {
	ios_base::sync_with_stdio(false);
#ifdef LOCAL
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#endif
	cin.tie(0);
}

const int base = 29;
int m;

ll prefHash[N];
ll suffHash[N];
ll rev;
int stringSize;

bool IsPrime(int t) {
	for (int i = 2; i * i <= t; i++) {
		if (t % i == 0)
			return false;
	}
	return true;
}

void InitHash(int n) {
	vector <int> hashes = { 1073742073, 1073743141, 1073743991, 1073744311, 1073744681 };
	m = hashes[n % hashes.size()];
}

ll binpow(ll a, ll p, ll modulo) {
	ll ans = 1;
	while (p) {
		if (p & 1)
			ans = (ans * a) % modulo;
		a = (a * a) % modulo;
		p >>= 1;
	}
	return ans;
}

void CountPrefHash(string& s) {
	prefHash[0] = s[0] - 'a'+1;
	ll b = base;
	for (int i = 1; i < s.size(); i++) {
		prefHash[i] = (prefHash[i - 1] + b * (ll)(s[i] - 'a' + 1)) % m;
		b = (base * b) % m;
	}
}

ll GetPrefHash(int pos, int len) {
	len = min(len, stringSize - pos - 1);
	ll k = (prefHash[pos + len] - prefHash[pos]) * binpow(rev, pos + 1, m);
	return k % m;
}

void CountSuffHash(string& s) {
	suffHash[s.size() - 1] = s[s.size() - 1] - 'a'+1;
	ll b = base;
	for (int i = (int)s.size() - 2; i >= 0; i--) {
		suffHash[i] = (suffHash[i + 1] + b * (ll)(s[i] - 'a' + 1)) % m;
		b = (base * b) % m;
	}
}

ll GetSuffHash(int pos, int len) {
	len = min(len, pos);
	return ((suffHash[pos - len] - suffHash[pos]) * binpow(rev, stringSize - pos, m)) % m;
}

void Init(string& s) {
	InitHash(s.size());
	stringSize = s.size();
	CountPrefHash(s);
	CountSuffHash(s);
	rev = binpow(29, m - 2, m);
}

void Solve() {
	string s;
	cin >> s >> s;
	Init(s);
	vector <set <int>> palindL(s.size()), palindR(s.size());
	for (int i = 0; i < s.size(); i++) {
		int l = 0, r = min((int)s.size() - i - 1, i) + 1;
		while (l < r - 1) {
			int d = (l + r) / 2;
			if (GetPrefHash(i, d) == GetSuffHash(i, d)) l = d;
			else r = d;
		}
		for (int j = 0; j < r; j++) {
			palindL[j + i].insert(1 + 2 * j);
			palindR[i - j].insert(1 + 2 * j);
		}
	}
	for (int i = 1; i < s.size(); i++) {
		int l = 0, r = min((int)s.size() - i, i) + 1;
		while (l < r - 1) {
			int d = (l + r) / 2;
			if (GetPrefHash(i-1, d) == GetSuffHash(i, d)) l = d;
			else r = d;
		}
		for (int j = 1; j < r; j++) {
			palindL[j + i - 1].insert(2 * j);
			palindR[i - j].insert(2 * j);
		}
	}
	ll ans = 0;
	for (int i = 1; i < s.size(); i++) {
		for (auto j : palindL[i - 1])
			if (palindR[i].count(j))
				ans++;
	}
	cout << ans;
}

int main() {
	IOOptimize();
	Solve();
}