﻿#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <set>
#include <map>

using namespace std;
using ll = long long;
const ll p = 10007;
const ll mod = 1000000007;
const int maxn = 500001;

ll pw[maxn];

void prec(int n) {
	pw[0] = 1;
	for (int i = 1; i <= n; i++) {
		pw[i] = pw[i - 1] * p % mod;
	}
}

vector<ll> build_h(string s) {
	int n = (int)s.size();
	vector<ll> h(n + 1);
	for (int i = 0; i < n; i++) {
		h[i + 1] = (h[i] * p + (s[i] - 'a' + 1)) % mod;
	}
	return h;
}

vector<ll> build_rh(string s) {
	int n = (int)s.size();
	vector<ll> h(n + 1);
	for (int i = n - 1; i >= 0; i--) {
		h[i] = (h[i + 1] * p + (s[i] - 'a' + 1)) % mod;
	}
	return h;
}

ll get_hash(vector<ll>& h, int l, int r) {
	return (h[r + 1] - h[l] * pw[r - l + 1] % mod + mod) % mod;
}

ll get_rhash(vector<ll>& h, int l, int r) {
	return (h[l] - h[r + 1] * pw[r - l + 1] % mod + mod) % mod;
}

int find_max_pal_odd(int i, vector<ll>& h1, vector<ll>& h2) {
	int n = (int)h1.size() - 1;
	int l = 0, r = n;
	while (r - l > 1) {
		int m = (l + r) / 2;
		int left = i - m, right = i + m;
		if (left < 0 || right >= n) {
			r = m;
			continue;
		}
		if (get_hash(h1, left, right) == get_rhash(h2, left, right)) {
			l = m;
		}
		else {
			r = m;
		}
	}
	return l;
}

int find_max_pal_even(int i, vector<ll>& h1, vector<ll>& h2) {
	int n = (int)h1.size() - 1;
	int l = 0, r = n;
	while (r - l > 1) {
		int m = (l + r) / 2;
		int left = i - m, right = i + m - 1;
		if (left < 0 || right >= n) {
			r = m;
			continue;
		}
		if (get_hash(h1, left, right) == get_rhash(h2, left, right)) {
			l = m;
		}
		else {
			r = m;
		}
	}
	return l;
}

bool cmp(pair<int, int> a, pair<int, int> b) {
	return make_pair(a.second, a.first) < make_pair(b.second, b.first);
}

//ll number_intersections(vector<pair<int, int> > seg0, vector<pair<int, int> > seg1, int same) {
//	sort(seg0.begin(), seg0.end(), cmp);
//	sort(seg1.begin(), seg1.end(), cmp);
//	ll ans = 0;
//	for (int i = 0; i < (int)seg0.size(); i++) {
//		int down = seg0[i].first - 1, up = seg0[i].second;
//		int left, right;
//		if (same) {
//			left = lower_bound(seg0.begin(), seg0.end(), make_pair(-1, down), cmp) - seg0.begin();
//			right = lower_bound(seg0.begin(), seg0.end(), seg0[i], cmp) - seg0.begin();
//		}
//		else {
//			left = lower_bound(seg1.begin(), seg1.end(), make_pair(-1, down), cmp) - seg1.begin();
//			right = lower_bound(seg1.begin(), seg1.end(), seg0[i], cmp) - seg1.begin();
//		}
//		ans += right - left;
//	}
//	for (int i = 0; i < (int)seg1.size(); i++) {
//		int down = seg1[i].first - 1, up = seg1[i].second;
//		int left, right;
//		if (same) {
//			left = lower_bound(seg1.begin(), seg1.end(), make_pair(-1, down), cmp) - seg1.begin();
//			right = lower_bound(seg1.begin(), seg1.end(), seg1[i], cmp) - seg1.begin();
//		}
//		else {
//			left = lower_bound(seg0.begin(), seg0.end(), make_pair(-1, down), cmp) - seg0.begin();
//			right = lower_bound(seg0.begin(), seg0.end(), seg1[i], cmp) - seg0.begin();
//		}
//		ans += right - left;
//	}
//	return ans;
//}

int main()
{
	int n;
	cin >> n;
	prec(n);
	string s;
	cin >> s;
	auto h1 = build_h(s);
	auto h2 = build_rh(s);
	vector<int> lens1(n);
	for (int i = 0; i < n; i++) {
		lens1[i] = find_max_pal_odd(i, h1, h2);
	}
	vector<int> lens2(n + 1);
	for (int i = 0; i <= n; i++) {
		lens2[i] = find_max_pal_even(i, h1, h2);
	}
	vector<int> b1(n), b2(n + 1);
	for (int i = 0; i < n; i++) {
		b1[i] = lens1[i] * 2;
	}
	for (int i = 0; i < n + 1; i++) {
		b2[i] = lens2[i] * 2;
	}
	for (int i = 0; i < n; i++) cout << b1[i] << ' ';
	cout << endl;
	ll ans = 0;
	for (int i = 0; i < n; i++) {
		for (int j = i + 1; j < n; j += 2) {
			if (j - i - 1 <= min(b1[i], b1[j])) {
				ans++;
				//cout << i << ' ' << j << endl;
			}
		}
	}
	for (int i = 0; i <= n; i++) {
		for (int j = i + 2; j <= n; j += 2) {
			if (j - i <= min(b2[i], b2[j])) {
				ans++;
			}
		}
	}
	cout << ans << endl;
}
