#include <iostream>
#include <vector>
#include <map>
#include <algorithm>
#include <cassert>

using namespace std;

struct STR {
	vector <long long> hashes, pows_p, rev_h;
	long long Q = 2286661337, p = 239;
	STR(string s = "a", long long Q = 2286661337, long long p = 239) : Q(Q), p(p) {
		hashes.resize(s.size() + 1);
		pows_p.resize(s.size() + 1);
		rev_h.resize(s.size() + 1);
		rev_h.back() = 0;
		hashes[0] = 0, pows_p[0] = 1;
		for (int i = 0; i < s.size(); i++) {
			hashes[i + 1] = (hashes[i] * p + s[i]) % Q;
			pows_p[i + 1] = pows_p[i] * p % Q;
		}
		for (int i = s.size() - 1; i > -1; i--) {
			rev_h[i] = (rev_h[i + 1] * p + s[i]) % Q;
		}
	}
	long long get_hash(int L, int R) {
		long long ans = hashes[R] - hashes[L] * pows_p[R - L] % Q;
		if (ans < 0)
			ans += Q;
		return ans;
	}
	long long get_rev_hash(int L, int R) {
		long long ans = rev_h[L] - rev_h[R] * pows_p[R - L] % Q;
		if (ans < 0)
			ans += Q;
		return ans;
	}
};

bool is_pal(int L, int R, STR& s) {
	return s.get_hash(L, L + (R - L) / 2) == s.get_rev_hash(R - (R - L) / 2, R);
}

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	int n;
	cin >> n;
	string tmp;
	cin >> tmp;
	string new_s;
	for (int i = 0; i < n; i++) {
		new_s += tmp[i];
		if (i != n - 1)
			new_s += '*';
	}
	n = new_s.size();
	tmp = new_s;
	STR s = STR(tmp);
	vector <int> cen(n, 0);
	for (int i = 0; i < n; i++) {
		int L = 1, R = min(i, n - i - 1) + 2;
		while (L + 1 < R) {
			int M = (L + R) / 2;
			if (s.get_hash(i, i + M) == s.get_rev_hash(i - M + 1, i + 1))
				L = M;
			else
				R = M;
		}
		cen[i] = L;
	}
	int ans = 0;
	for (int i = 0; i < n; i++) {
		if (i % 2) {
			for (int offset = 4; offset <= cen[i] * 2 && i + offset < n; offset += 4) {
				if (cen[i + offset] >= offset / 2)
					ans++;
			}
		}
		else {
			for (int offset = 2; offset <= cen[i] * 2 && i + offset < n; offset += 4) {
				if (cen[i + offset] >= offset / 2)
					ans++;
			}
		}
	}
	for (int i = 0; i < n; i++) {
		assert(is_pal(i - cen[i] + 1, i + cen[i], s));
	}
	cout << ans;
}