#include<iostream>
#include<cstring>
#include<vector>

using namespace std;

const int MAXN = 5e5 + 5;
const pair <long long, long long> key = {29, 31};
const pair <long long, long long> mod = {1e9+7, 1e9+9};

struct FenwickTree
{
    int tree[MAXN];

    FenwickTree()
    {
        memset(this->tree, 0, sizeof(this->tree));
    }

    void reset()
    {
        memset(this->tree, 0, sizeof(this->tree));
    }

    void update(int ind, int val)
    {
        ind++;
        while(ind<MAXN)
        {
            this->tree[ind] += val;
            ind += ind&(-ind);
        }
    }

    int query(int ind)
    {
        ind++;
        int sum = 0;

        while(ind>0)
        {
            sum += this->tree[ind];
            ind -= ind&(-ind);
        }

        return sum;
    }

    int query(int l, int r)
    {
        return query(r) - query(l-1);
    }
};

struct Query
{
    int x;
    int parity, sign;

    Query(){}
    Query(int x, int parity, int sign)
    {
        this->x = x;
        this->sign = sign;
        this->parity = parity;
    }
};

int n;
string s;

pair <long long, long long> p[MAXN];
pair <long long, long long> h[MAXN], hRev[MAXN];

int palSize[MAXN];
vector <Query> q[MAXN];

void Init()
{
    p[0] = {1, 1};
    for(int i = 1;i<=n;i++)
    {
        p[i].first = (p[i-1].first*key.first)%mod.first;
        p[i].second = (p[i-1].second*key.second)%mod.second;
    }

    h[0] = {s[0]-'a'+1, s[0]-'a'+1};
    for(int i = 1;i<n;i++)
    {
        h[i].first = (h[i-1].first*key.first+(s[i]-'a'+1))%mod.first;
        h[i].second = (h[i-1].second*key.second+(s[i]-'a'+1))%mod.second;
    }

    hRev[n-1] = {s[n-1]-'a'+1, s[n-1]-'a'+1};
    for(int i = n-2;i>=0;i--)
    {
        hRev[i].first = (hRev[i+1].first*key.first+(s[i]-'a'+1))%mod.first;
        hRev[i].second = (hRev[i+1].second*key.second+(s[i]-'a'+1))%mod.second;
    }
}

pair <long long, long long> getHash(int l, int r)
{
    if(l==0) return h[r];
    return {(h[r].first-h[l-1].first*p[r-l+1].first+mod.first*mod.first)%mod.first,
            (h[r].second-h[l-1].second*p[r-l+1].second+mod.second*mod.second)%mod.second};
}

pair <long long, long long> getHashRev(int l, int r)
{
    if(r==n-1) return hRev[l];
    return {(hRev[l].first-hRev[r+1].first*p[r-l+1].first+mod.first*mod.first)%mod.first,
            (hRev[l].second-hRev[r+1].second*p[r-l+1].second+mod.second*mod.second)%mod.second};
}

FenwickTree T[2];
long long solveOdd()
{
    T[0].reset();
    T[1].reset();
    for(int i = 0;i<n;i++)
    {
        q[i].clear();
    }

    long long answer = 0;
    for(int i = 0;i<n;i++)
    {
        int l = 1, r = min(i+1, n-i), mid;
        while(l+1<r)
        {
            mid = (l+r)/2;

            if(getHash(i-mid+1, i)==getHashRev(i, i+mid-1)) l = mid;
            else r = mid - 1;
        }

        if(getHash(i-r+1, i)==getHashRev(i, i+r-1)) palSize[i] = r;
        else palSize[i] = l;
    }

    for(int i = 1;i<n;i++)
    {
        //cout << i << " " << palSize[i] << '\n';

        q[i-1].push_back(Query(i, (i+1)%2, +1));
        if((i-palSize[i]*2+1)-1>=0) q[(i-palSize[i]*2+1)-1].push_back(Query(i, (i+1)%2, -1));
    }
    for(int i = 0;i<n;i++)
    {
        //cout << "update: " << i << " " << i+palSize[i]*2-1 << '\n';

        T[i%2].update(min(n-1, i+palSize[i]*2-1), +1);
        for(Query item: q[i])
        {
            //cout << item.x << " " << item.sign << " -> " << T[item.parity].query(item.x, n-1) << '\n';
            answer += T[item.parity].query(item.x, n-1)*item.sign;
        }

        //cout << " -> " << answer << '\n';
    }
    /*
        //cout << " --- " << i << " --- " << '\n';
        for(int j = 0;j<i;j++)
        {
            if(i-palSize[i]*2+1>j) continue;
            if(i%2==j%2) continue;


            if(j+palSize[j]*2-1>=i)
            {
                answer++;
                //cout << " -> " << j << '\n';
            }
        }
    }
    */

    return answer;
}

long long solveEven()
{
    T[0].reset();
    T[1].reset();
    for(int i = 0;i<n;i++)
    {
        q[i].clear();
    }

    long long answer = 0;
    for(int i = 0;i<n-1;i++)
    {
        int l = 0, r = min(i+1, n-i-1), mid;
        while(l+1<r)
        {
            mid = (l+r)/2;

            if(getHash(i-mid+1, i)==getHashRev(i+1, i+1+mid-1)) l = mid;
            else r = mid - 1;
        }

        if(getHash(i-r+1, i)==getHashRev(i+1, i+1+r-1)) palSize[i] = r;
        else palSize[i] = l;

        //cout << i << " -> " << palSize[i] << '\n';
    }

    for(int i = 2;i<n-1;i++)
    {
        if(palSize[i]==0) continue;

        q[i-2].push_back(Query(i, i%2, +1));
        if((i-palSize[i]*2+1-1)-1>=0) q[(i-palSize[i]*2+1-1)-1].push_back(Query(i, i%2, -1));
    }

    for(int i = 0;i<n;i++)
    {
        T[i%2].update(min(n-1, i+1+palSize[i]*2-1), +1);
        for(Query item: q[i])
        {
            //cout << item.x << " " << item.sign << " -> " << T[item.parity].query(item.x, n-1) << '\n';
            answer += T[item.parity].query(item.x, n-1)*item.sign;
        }
    }

    /*
    for(int i = 0;i<n-1;i++)
    {
        //cout << " --- " << i << " --- " << '\n';
        for(int j = 0;j<i;j++)
        {
            if(i-palSize[i]*2+1-1>j) continue;
            if(i%2!=j%2) continue;

            if(j+1+palSize[j]*2-1>=i)
            {
                answer++;
                //cout << " -> " << j << '\n';
            }
        }
    }
    */

    return answer;
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    long long answer = 0;

    cin >> n;
    cin >> s;

    Init();
    answer += solveOdd();
    answer += solveEven();

    cout << answer << '\n';
}
