﻿#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <set>
#include <map>
#pragma GCC optimize("Ofast")

using namespace std;
using ll = long long;
const ll p = 10007;
const ll mod = 1000000007;
const int maxn = 500001;

ll pw[maxn];

void prec(int n) {
	pw[0] = 1;
	for (int i = 1; i <= n; i++) {
		pw[i] = pw[i - 1] * p % mod;
	}
}

vector<ll> build_h(string s) {
	int n = (int)s.size();
	vector<ll> h(n + 1);
	for (int i = 0; i < n; i++) {
		h[i + 1] = (h[i] * p + (s[i] - 'a' + 1)) % mod;
	}
	return h;
}

vector<ll> build_rh(string s) {
	int n = (int)s.size();
	vector<ll> h(n + 1);
	for (int i = n - 1; i >= 0; i--) {
		h[i] = (h[i + 1] * p + (s[i] - 'a' + 1)) % mod;
	}
	return h;
}

ll get_hash(vector<ll>& h, int l, int r) {
	return (h[r + 1] - h[l] * pw[r - l + 1] % mod + mod) % mod;
}

ll get_rhash(vector<ll>& h, int l, int r) {
	return (h[l] - h[r + 1] * pw[r - l + 1] % mod + mod) % mod;
}

int find_max_pal_odd(int i, vector<ll>& h1, vector<ll>& h2) {
	int n = (int)h1.size() - 1;
	int l = 0, r = n;
	while (r - l > 1) {
		int m = (l + r) / 2;
		int left = i - m, right = i + m;
		if (left < 0 || right >= n) {
			r = m;
			continue;
		}
		if (get_hash(h1, left, right) == get_rhash(h2, left, right)) {
			l = m;
		}
		else {
			r = m;
		}
	}
	return l;
}

int find_max_pal_even(int i, vector<ll>& h1, vector<ll>& h2) {
	int n = (int)h1.size() - 1;
	int l = 0, r = n;
	while (r - l > 1) {
		int m = (l + r) / 2;
		int left = i - m, right = i + m - 1;
		if (left < 0 || right >= n) {
			r = m;
			continue;
		}
		if (get_hash(h1, left, right) == get_rhash(h2, left, right)) {
			l = m;
		}
		else {
			r = m;
		}
	}
	return l;
}

struct stree {
	vector<int> t;
	void init(int n) {
		int _n = 1;
		while (_n < n) _n *= 2;
		n = _n;
		t.assign(2 * n, 0);
	}
	void update(int i, int x) {
		i += (int)t.size() / 2;
		t[i] = x;
		while (i > 1) {
			i /= 2;
			t[i] = t[2 * i] + t[2 * i + 1];
		}
	}
	int sum(int l, int r) {
		l += (int)t.size() / 2, r += (int)t.size() / 2;
		int ans = 0;
		while (l < r) {
			if (l & 1) ans += t[l++];
			if (r & 1) ans += t[--r];
			l /= 2, r /= 2;
		}
		return ans;
	}
};

int main()
{
	ios::sync_with_stdio(0);
	int n;
	cin >> n;
	prec(n);
	string s;
	cin >> s;
	auto h1 = build_h(s);
	auto h2 = build_rh(s);
	vector<int> lens1(n);
	for (int i = 0; i < n; i++) {
		lens1[i] = find_max_pal_odd(i, h1, h2);
	}
	vector<int> lens2(n + 1);
	for (int i = 0; i <= n; i++) {
		lens2[i] = find_max_pal_even(i, h1, h2);
	}
	vector<int> b1(n), b2(n + 1);
	for (int i = 0; i < n; i++) {
		b1[i] = lens1[i] * 2;
	}
	for (int i = 0; i < n + 1; i++) {
		b2[i] = lens2[i] * 2;
	}
	/*for (int i = 0; i < n; i++) cout << b1[i] << ' ';
	cout << endl;*/
	ll ans = 0;
	//vector<int> cur(n);
	set<pair<int, int> > s1;
	stree t0, t1;
	t0.init(n), t1.init(n);
	for (int i = 0; i < n; i++) {
		/*int sum = 0;
		for (int j = i - 1; j >= max(0, i - b1[i] - 1); j -= 2) sum += cur[j];
		ans += sum;*/
		if (i % 2 == 0) ans += t1.sum(max(0, i - b1[i] - 1), i);
		else ans += t0.sum(max(0, i - b1[i] - 1), i);
		s1.insert({ i + b1[i] + 1, i });
		if (i % 2 == 0) t0.update(i, 1);
		else t1.update(i, 1);
		while (!s1.empty() && (*s1.begin()).first <= i) {
			auto elem = *s1.begin();
			s1.erase(s1.begin());
			if (elem.second % 2 == 0) t0.update(elem.second, 0);
			else t1.update(elem.second, 0);
		}
	}
	//cout << ans << endl;
	s1.clear();
	t1.init(n + 1), t0.init(n + 1);
	for (int i = 0; i <= n; i++) {
		/*int sum = 0;
		for (int j = i - 2; j >= max(0, i - b2[i]); j -= 2) sum += cur[j];
		ans += sum;*/
		if (i % 2 == 1) ans += t1.sum(max(0, i - b2[i]), i - 1);
		else ans += t0.sum(max(0, i - b2[i]), i - 1);
		s1.insert({ i + b2[i], i });
		if (i % 2 == 0) t0.update(i, 1);
		else t1.update(i, 1);
		while (!s1.empty() && (*s1.begin()).first <= i) {
			auto elem = *s1.begin();
			s1.erase(s1.begin());
			if (elem.second % 2 == 0) t0.update(elem.second, 0);
			else t1.update(elem.second, 0);
		}
	}
	cout << ans << endl;
	//for (int i = 0; i < n; i++) {
	//	for (int j = i + 1; j < n; j += 2) {
	//		if (j - i - 1 <= min(b1[i], b1[j])) {
	//			ans++;
	//			//cout << i << ' ' << j << endl;
	//		}
	//	}
	//}
	//for (int i = 0; i <= n; i++) {
	//	for (int j = i + 2; j <= n; j += 2) {
	//		if (j - i <= min(b2[i], b2[j])) {
	//			ans++;
	//		}
	//	}
	//}
	//cout << ans << endl;
}
