#include <iostream>
#include <vector>
#include <algorithm>
#include <set>

using namespace std;

typedef long long ll;

const ll BASE = 31;
const ll MOD = 1200720071;

int n;
vector<ll> BASES;

vector<ll> hash_string(string &s)
{
    int n = s.size();
    vector<ll> h(n + 1, 0);

    for (int i = 0; i < n; i++)
        h[i + 1] = ((h[i] * BASE) % MOD + s[i] - 'a' + 1) % MOD;

    return h;
}

ll substring_hash(vector<ll> &h, int l, int r)
{
    return ((h[r] - (h[l] * BASES[r - l]) % MOD) % MOD + MOD) % MOD;
}

bool is_palindrome(vector<ll> &fwd, vector<ll> &rev, int l, int r)
{
    if (l < 0 || r < 0 || r >= n || l >= n || l > r)
        return false;
    else
        return substring_hash(fwd, l, r + 1) ==
               substring_hash(rev, n - r - 1, n - l);
}

const int MAXN = 2e6;

int val[MAXN];

void build(int t, int l, int r, vector<int> &a)
{
    if (l == r - 1)
    {
        val[t] = a[l];
        return;
    }

    int m = (l + r) / 2;
    build(2 * t, l, m, a);
    build(2 * t + 1, m, r, a);
    val[t] = val[2 * t] + val[2 * t + 1];
}

int get(int t, int l, int r, int L, int R)
{
    if (l >= r || L >= R || L < l || R > r)
        return 0;
    if (l == L && r == R)
        return val[t];

    int m = (l + r) / 2;
    return get(2 * t, l, m, L, min(m, R)) +
           get(2 * t + 1, m, r, max(L, m), R);
}

void update(int t, int l, int r, int i, int x)
{
    if (l >= r)
        return;
    if (l == r - 1)
    {
        val[t] = x;
        return;
    }

    int m = (l + r) / 2;
    if (i < m)
        update(2 * t, l, m, i, x);
    else
        update(2 * t + 1, m, r, i, x);
    val[t] = val[2 * t] + val[2 * t + 1];
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    BASES.resize(1e6, 1);
    for (int i = 1; i < 1e6; i++)
        BASES[i] = (BASES[i - 1] * BASE) % MOD;

    string s;
    cin >> n >> s;

    auto fwd = hash_string(s);
    reverse(s.begin(), s.end());
    auto rev = hash_string(s);

    vector<ll> max_odd(n, 0);
    for (int i = 0; i < n; i++)
    {
        int l = 0;
        int r = n;
        while (r - l > 1)
        {
            int m = (l + r) / 2;
            if (is_palindrome(fwd, rev, i - m, i + m))
                l = m;
            else
                r = m;
        }

        max_odd[i] = l;
    }

    vector<ll> max_even(n, 0);
    for (int i = 0; i < n; i++)
    {
        int l = 0;
        int r = n;
        while (r - l > 1)
        {
            int m = (l + r) / 2;
            if (is_palindrome(fwd, rev, i - m, i + m - 1))
                l = m;
            else
                r = m;
        }

        max_even[i] = l;
    }

//    for (ll x : max_odd)
//        cout << x << " ";
//    cout << endl;
//    for (ll x : max_even)
//        cout << x << " ";
//    cout << endl;

    ll ans = 0;

    // odd pali, odd/even
    vector<int> b(n, 0);
    build(1, 0, n, b);
    vector<vector<int>> add_for(n);
    for (int i = 0; i < n; i += 2)
        add_for[max(0ll, i - 2 * max_odd[i] - 1)].push_back(i);
    for (int i = 1; i < n; i += 2)
    {
        for (int x : add_for[i - 1])
            update(1, 0, n, x, 1);
        for (int x : add_for[i])
            update(1, 0, n, x, 1);

        ans += (ll)get(1, 0, n, i + 1, min((ll)n, i + 2 * max_odd[i] + 2));
    }

    // odd pali, even/odd
    build(1, 0, n, b);
    add_for.clear();
    add_for.resize(n);
    for (int i = 1; i < n; i += 2)
        add_for[max(0ll, i - 2 * max_odd[i] - 1)].push_back(i);
    for (int i = 0; i < n; i += 2)
    {
        if (i != 0)
        {
            for (int x : add_for[i - 1])
                update(1, 0, n, x, 1);
        }
        for (int x : add_for[i])
            update(1, 0, n, x, 1);

        ans += (ll)get(1, 0, n, i + 1, min((ll)n, i + 2 * max_odd[i] + 2));
    }


    // even pali, odd/odd
    build(1, 0, n, b);
    add_for.clear();
    add_for.resize(n);
    for (int i = 1; i < n; i += 2)
        add_for[max(0ll, i - 2 * max_even[i])].push_back(i);

    for (int i = 1; i < n; i += 2)
    {
        for (int x : add_for[i - 1])
            update(1, 0, n, x, 1);
        for (int x : add_for[i])
            update(1, 0, n, x, 1);

        ans += (ll)get(1, 0, n, i + 2, min((ll)n, i + 2 * max_even[i] + 1));
    }

    // even pali, even/even
    build(1, 0, n, b);
    add_for.clear();
    add_for.resize(n);
    for (int i = 0; i < n; i += 2)
        add_for[max(0ll, i - 2 * max_even[i])].push_back(i);

    for (int i = 0; i < n; i += 2)
    {
        if (i != 0)
        {
            for (int x : add_for[i - 1])
                update(1, 0, n, x, 1);
        }
        for (int x : add_for[i])
            update(1, 0, n, x, 1);

        ans += (ll)get(1, 0, n, i + 2, min((ll)n, i + 2 * max_even[i] + 1));
    }

    cout << ans << endl;

    return 0;
}