#include <iostream>
#include <vector>
#include <map>
#include <algorithm>
#include <cassert>

using namespace std;

struct STR {
	vector <long long> hashes, pows_p, rev_h;
	long long Q = 2286661337, p = 239;
	STR(string s = "a", long long Q = 2286661337, long long p = 239) : Q(Q), p(p) {
		hashes.resize(s.size() + 1);
		pows_p.resize(s.size() + 1);
		rev_h.resize(s.size() + 1);
		rev_h.back() = 0;
		hashes[0] = 0, pows_p[0] = 1;
		for (int i = 0; i < s.size(); i++) {
			hashes[i + 1] = (hashes[i] * p + s[i]) % Q;
			pows_p[i + 1] = pows_p[i] * p % Q;
		}
		for (int i = s.size() - 1; i > -1; i--) {
			rev_h[i] = (rev_h[i + 1] * p + s[i]) % Q;
		}
	}
	long long get_hash(int L, int R) {
		long long ans = hashes[R] - hashes[L] * pows_p[R - L] % Q;
		if (ans < 0)
			ans += Q;
		return ans;
	}
	long long get_rev_hash(int L, int R) {
		long long ans = rev_h[L] - rev_h[R] * pows_p[R - L] % Q;
		if (ans < 0)
			ans += Q;
		return ans;
	}
};

struct Tree {
	vector <vector <int>> tree;
	void build(int v, int vL, int vR, vector <int>& a) {
		if (vL + 1 >= vR) {
			tree[v] = { a[vL] };
			return;
		}
		int vM = (vL + vR) / 2;
		build(2 * v, vL, vM, a);
		build(2 * v + 1, vM, vR, a);
		tree[v].resize(tree[2 * v].size() + tree[2 * v + 1].size());
		merge(tree[2 * v].begin(), tree[2 * v].end(), 
			tree[2 * v + 1].begin(), tree[2 * v + 1].end(), tree[v].begin());
	}
	Tree(vector <int> a = { 0 }) {
		tree.resize(4 * a.size());
		if (a.size()) build(1, 0, a.size(), a);
	}
	int cnt_more(int v, int vL, int vR, int L, int R, int x) {
		if (R <= vL || vR <= L)
			return 0;
		if (L <= vL && vR <= R)
			return tree[v].end() - upper_bound(tree[v].begin(), tree[v].end(), x);
		int vM = (vL + vR) / 2;
		return cnt_more(2 * v, vL, vM, L, R, x) + cnt_more(2 * v + 1, vM, vR, L, R, x);
	}
	int cnt_more(int L, int R, int x) {
		return (tree.size() ? cnt_more(1, 0, tree.size() / 4, L, R, x) : 0);
	}
};

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	int n;
	cin >> n;
	string tmp;
	cin >> tmp;
	string new_s;
	for (int i = 0; i < n; i++) {
		new_s += tmp[i];
		if (i != n - 1)
			new_s += '*';
	}
	n = new_s.size();
	tmp = new_s;
	STR s = STR(tmp);
	vector <int> cen(n, 0);
	for (int i = 0; i < n; i++) {
		int L = 1, R = min(i, n - i - 1) + 2;
		while (L + 1 < R) {
			int M = (L + R) / 2;
			if (s.get_hash(i, i + M) == s.get_rev_hash(i - M + 1, i + 1))
				L = M;
			else
				R = M;
		}
		cen[i] = L * 2;
	}
	vector <int> for_tree[4];
	for (int i = 0; i < n; i++) {
		for_tree[i & 3].emplace_back(cen[i] - i);
	}
	Tree tr[4];
	for (int i = 0; i < 4; i++) {
		tr[i] = Tree(for_tree[i]);
	}
	int ans = 0;
	for (int i = 0; i < n; i++) {
		if (i % 2) {
			ans += tr[i & 3].cnt_more(i / 4 + 1, i / 4 + cen[i] / 4 + 1, -i - 1);
			/*for (int offset = 4; offset <= cen[i] && i + offset < n; offset += 4) {
				if (cen[i + offset] >= offset)
					ans++;
			}*/
		}
		else {
			ans += tr[(i + 2) & 3].cnt_more((i + 2) / 4, (i + 2) / 4 + cen[i] / 4 + 1, -i - 1);
			/*for (int offset = 2; offset <= cen[i] && i + offset < n; offset += 4) {
				if (cen[i + offset] >= offset)
					ans++;
			}*/
		}
	}
	cout << ans;
}