#include <iostream>
#include <cstdint>
#include <vector>
#include <algorithm>
#include <cstring>
using namespace std;

static const uint32_t K[]={13, 17, 23};
static const uint32_t MOD[]={1000000007ULL, 837133713ULL, 945682313ULL};

#define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))

struct hash_mem_data
{
    uint32_t mk[ARRAY_SIZE(K)];
};

vector<hash_mem_data> pow_mem;

static inline void pow_mem_init(uint32_t n)
{
    n++;
    pow_mem.resize(n);
    uint64_t tmp[ARRAY_SIZE(K)];
    for(uint32_t i=0; i<ARRAY_SIZE(K); i++)
        tmp[i]=1;

    for(uint32_t i=0; i<n; i++)
    {
        for(uint32_t j=0; j<ARRAY_SIZE(K); j++)
        {
            pow_mem[i].mk[j]=(uint32_t)tmp[j];
            tmp[j]*=K[j];
            tmp[j]%=MOD[j];
        }
    }
}

class hash_data
{
private:
    uint32_t hd[3];

public:

    void make_zero()
    {
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
            hd[i]=0;
    }

    void make_one()
    {
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
            hd[i]=1;
    }

    template <class T>
    hash_data append(T x, uint32_t n)
    {
        uint64_t tmp;
        hash_data res;
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
        {
            tmp=x;
            tmp*=pow_mem[n].mk[i];
            tmp%=MOD[i];
            tmp+=hd[i];
            tmp%=MOD[i];
            res.hd[i]=(uint32_t)tmp;
        }

        return res;
    }

    hash_data pow(uint32_t n)
    {
        uint64_t tmp;
        hash_data res;
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
        {
            tmp=hd[i];
            tmp*=pow_mem[n].mk[i];
            tmp%=MOD[i];
            res.hd[i]=(uint32_t)tmp;
        }

        return res;
    }

    hash_data operator- (const hash_data b)
    {
        hash_data res;
        int64_t tmp;
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
        {
            tmp=hd[i];
            tmp-=b.hd[i];
            if(tmp<0)
                tmp+=MOD[i];
            res.hd[i]=(uint32_t)tmp;
        }

        return res;
    }

    hash_data operator+ (const hash_data b)
    {
        hash_data res;
        int64_t tmp;
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
        {
            tmp=hd[i];
            tmp+=b.hd[i];
            tmp%=MOD[i];
            res.hd[i]=(uint32_t)tmp;
        }

        return res;
    }

    bool operator== (const hash_data b)
    {
        for(uint32_t i=0; i<ARRAY_SIZE(hd); i++)
        {
            if(hd[i]!=b.hd[i])
                return false;
        }
        return true;
    }
};

class hash_pref_data
{
private:
    vector<hash_data> pref;

public:

    template <class T>
    void init(const vector<T> &data)
    {
        pref.resize(data.size()+1);
//        hash_data curh;
//        curh.make_one();
        pref[0].make_zero();
        for(uint32_t i=0; i<data.size(); i++)
        {
            //curh=curh.append(data[i], i);
            //pref[i+1]=pref[i]+curh;
            pref[i+1]=pref[i].append(data[i], i);
        }
    }

    /**
     *
     * @param b
     * @param e
     *
     * @note interval is [a, b]
     */
    hash_data get_subhash (uint32_t b, uint32_t e)
    {
        e++;
        hash_data res;
        res = pref[e]-pref[b];
        uint32_t tmpp=pref.size()-b;
        res = res.pow(tmpp);
        return res;
    }
};

uint32_t n;
hash_pref_data hps, hprs;

static inline bool is_pallind (uint32_t pb, uint32_t pe)
{
    if(pb==pe)
        return true;
    hash_data hp1, hp2;
    uint32_t sized2 = pe-pb+1;
    sized2/=2;
    sized2--;
    hp1=hps.get_subhash(pb, pb+sized2);
    hp2=hprs.get_subhash((n-1)-pe, (n-1)-(pe-sized2));
    return hp1==hp2;
}

vector< vector< bool > > is_pallind_mem;

static inline void init()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cin>>n;
    pow_mem_init(n + 16);

    vector<uint8_t> s;
    s.resize(n);
    char x;

    for(uint32_t i=0; i<n; i++)
    {
        cin>>x;
        x-='a';
        x++;
        s[i]=x;
    }

    hps.init(s);

//    asm volatile ("nop");
//
//    hash_data tha, thb;
//    tha = hps.get_subhash(2, 3);
//    thb = hps.get_subhash(4, 5);
//
//    asm volatile ("nop");

    auto sr = s;
    reverse(sr.begin(), sr.end());

    hprs.init(sr);

    is_pallind_mem.resize(n);
    for(uint32_t i=0; i<n; i++)
    {
        is_pallind_mem[i].resize(n);
        for(uint32_t j=i; j<n; j++)
        {
            is_pallind_mem[i][j]=is_pallind(i, j);
//            if(is_pallind_mem[i][j])
//                cout<<"PALLIND: "<<i<<' '<<j<<endl;
        }
    }

//    cout<<is_pallind(0, 3)<<endl;

}

static inline bool is_dpalld (uint32_t b, uint32_t e)
{
    uint32_t mid = (b+e)>>1;
    return is_pallind_mem[b][mid] && is_pallind_mem[mid+1][e];
}

uint64_t solve()
{
    uint64_t res=0;
    for(uint32_t i=0; i<n; i++)
    {
        for(uint32_t j=i+1; j<n; j+=2)
        {
            if(is_dpalld(i, j))
                res++;
        }
    }

    return res;
}

int main()
{
    init();

    cout<<solve()<<endl;

    return 0;
}