#pragma region Include
#include<algorithm>
#include<iostream>
#include<assert.h>
#include<utility>
#include<fstream>
#include<vector>
#include<random>
#include<map>
using namespace std;
#pragma endregion
#pragma region Constant
const int N = 5e5, INF = 1e9, MOD = 1e9 + 21, P = 31;
const long long llINF = 1e18;
#pragma endregion
#pragma region IO
template<typename T>
istream& operator >> (istream& in, vector<T>& vec) {
	for (T& el : vec) {
		in >> el;
	}
	return in;
}

template<typename T>
ostream& operator << (ostream& out, vector<T> const& vec) {
	for (T const& el : vec) {
		out << el << ' ';
	}
	return out;
}

template<typename T, typename P>
istream& operator >> (istream& in, pair<T, P>& p) {
	in >> p.first >> p.second;
	return in;
}

template<typename T, typename P>
ostream& operator << (ostream& out, pair<T, P> const& p) {
	out << p.first << ' ' << p.second;
}
#pragma endregion

class RangeTree {
	int n;
	vector<int> t;

	void upd_t(int v, int l, int r, int p, int x) {
		if (p < l || r <= p) {
			return;
		}

		if (r - l == 1) {
			t[v] += x;
			return;
		}

		upd_t(2 * v, l, (l + r) / 2, p, x);
		upd_t(2 * v + 1, (l + r) / 2, r, p, x);
		t[v] = t[2 * v] + t[2 * v + 1];
	}

	int get_t(int v, int l, int r, int L, int R) {
		if (R <= l || r <= L) {
			return 0;
		}

		if (L <= l && r <= R) {
			return t[v];
		}

		return get_t(2 * v, l, (l + r) / 2, L, R) + get_t(2 * v + 1, (l + r) / 2, r, L, R);
	}

public:
	RangeTree(int n) : n(n), t(4 * n) { }

	void upd(int p, int x) {
		upd_t(1, 0, n, p, x);
	}

	int get_t(int l, int r) {
		return get_t(1, 0, n, l, r);
	}
};

struct act {
	int pos, t, a;
	bool operator < (act const& other) {
		if (pos != other.pos) {
			return pos < other.pos;
		}
		return t < other.t;
	}
};

int sum(int a, int b) {
	a += b;
	if (a >= MOD) {
		a -= MOD;
	}
	return a;
}

int sub(int a, int b) {
	a -= b;
	if (a < 0) {
		a += MOD;
	}
	return a;
}

int mul(long long a, long long b) {
	a *= b;
	if (a >= MOD) {
		a %= MOD;
	}
	return a;
}

int p_pow[N];

vector<int> h_f(string const& s) {
	vector<int> h(s.size() + 1);
	for (int i = 0; i < s.size(); i++) {
		h[i + 1] = sum(mul(h[i], P), (s[i] - 'a') + 1);
	}
	return h;
}

int h_sub(int l, int r, vector<int> const& h) {
	++l;
	++r;
	return sub(h[r], mul(h[l - 1], p_pow[r - l + 1]));
}

vector<int> calc_max_ev(string const& s) {
	vector<int> h = h_f(s);
	int n = s.size() - 1;
	vector<int> res(n);

	for (int i = 0; i < n; i++) {
		int l = 0, r = min(i + 1, n - i) + 1;
		while (l + 1 < r) {
			int m = (l + r) / 2;
			if (h_sub(i - m + 1, i, h) == h_sub(i + 1, i + m, h)) {
				l = m;
			} else {
				r = m;
			}
		}
		res[i] = l;
	}

	return res;
}

int main() {
	//ifstream cin("input.txt");
	//ofstream cout("output.txt");

	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int n;
	string s;
	cin >> n >> s;

	p_pow[0] = 1;
	for (int i = 1; i <= n; i++) {
		p_pow[i] = mul(P, p_pow[i - 1]);
	}

	vector<int> max_ev_pal = calc_max_ev(s), max_odd_pal(n);

	//for (int i = 0; i + 1 < n; i++) {
	//	for (int j = 0; i - j >= 0 && i + j + 1 < n; j++) {
	//		if (s[i - j] == s[i + j + 1]) {
	//			max_ev_pal[i] = j + 1;
	//		} else {
	//			break;
	//		}
	//	}
	//}

	for (int i = 0; i < n; i++) {
		for (int j = 1; i - j >= 0 && i + j < n; j++) {
			if (s[i - j] == s[i + j]) {
				max_odd_pal[i] = j;
			} else {
				break;
			}
		}
	}

	RangeTree odd_0(n), odd_1(n), ev_0(n), ev_1(n);
	vector<act> line;

	for (int i = 0; i < n; i++) {
		line.push_back({ i, 1, max_odd_pal[i] });
		line.push_back({ i + 2 * max_odd_pal[i] + 2, -1, i });
	}

	for (int i = 0; i + 1 < n; i++) {
		line.push_back({ i, 2, max_ev_pal[i] });
		line.push_back({ i + max_ev_pal[i] * 2 + 1, -2, i });
	}

	sort(line.begin(), line.end());
	long long ans = 0;

	for (auto [pos, t, a] : line) {
		if (t == -1) {
			if (a & 1) {
				odd_1.upd(a, -1);
			} else {
				odd_0.upd(a, -1);
			}
		} else if (t == -2) {
			if (a & 1) {
				ev_1.upd(a, -1);
			} else {
				ev_0.upd(a, -1);
			}
		} else if (t == 1) {
			if (pos & 1) {
				ans += odd_0.get_t(pos - 2 * a - 1, pos);
				odd_1.upd(pos, 1);
			} else {
				ans += odd_1.get_t(pos - 2 * a - 1, pos);
				odd_0.upd(pos, 1);
			}
		} else if (t == 2) {
			if (pos & 1) {
				ans += ev_1.get_t(pos - 2 * a, pos);
				ev_1.upd(pos, 1);
			} else {
				ans += ev_0.get_t(pos - 2 * a, pos);
				ev_0.upd(pos, 1);
			}
		}
	}
	
	cout << ans << '\n';
}