#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <math.h>
#include <deque>
#define ll long long
#define ld long double

using namespace std;

const int MAXN = 5e5 + 5;

int tree[MAXN];

void add(int x, int val) {
	for (; x < MAXN; x = x | (x + 1)) {
		tree[x] += val;
	}
}

int getS(int r) {
	int res = 0;
	for (; r >= 0; r = (r & (r + 1)) - 1)
		res += tree[r];
	return res;
}

int getS(int l, int r) {
	return getS(r) - getS(l - 1);
}

void del_all() {
	for (int i = 0; i < MAXN; i++)
		tree[i] = 0;
}

const ll mod1 = 1200720071;
const ll mod2 = 1791791791;
const ll x1 = 113;
const ll x2 = 43;
vector<ll> p1;
vector<ll> p2;

void precalc() {
	p1.resize(MAXN);
	p2.resize(MAXN);
	p1[0] = 1;
	p2[0] = 1;
	for (int i = 1; i < MAXN; i++) {
		p1[i] = p1[i - 1] * x1 % mod1;
		p2[i] = p2[i - 1] * x2 % mod2;
	}
}

ll geth1(vector<ll>& h, int L, int R, vector<ll>& p, ll mod) {
	ll hr = h[R + 1];
	ll hl = h[L] * p[R - L + 1] % mod;
	hr -= hl;
	hr = (hr % mod + mod) % mod;
	return hr;
}

ll geth2(vector<ll>& h, int L, int R, vector<ll>& p, ll mod) {
	ll hr = h[R + 1] * p[R - L + 1] % mod;
	ll hl = h[L];
	hl -= hr;
	hl = (hl % mod + mod) % mod;
	return hl;
}

bool check(int L, int R, vector<ll> &h1, vector<ll> &h2, vector<ll> &H1, vector<ll> &H2) {
	ll a1 = geth1(h1, L, R, p1, mod1);
	ll a2 = geth1(h2, L, R, p2, mod2);
	ll b1 = geth2(H1, L, R, p1, mod1);
	ll b2 = geth2(H2, L, R, p2, mod2);
	return (a1 == b1) && (a2 == b2);
}

int main() {
	precalc();
	int n;
	cin >> n;
	string s;
	cin >> s;
	vector<ll> h1(n + 1), h2(n + 1);
	for (int i = 1; i <= n; i++) {
		h1[i] = (h1[i - 1] * x1 + (s[i - 1] - 'a') + 1) % mod1;
		h2[i] = (h2[i - 1] * x2 + (s[i - 1] - 'a') + 1) % mod2;
	}
	vector<ll> H1(n + 1), H2(n + 1);
	for (int i = n - 1; i >= 0; i--) {
		H1[i] = (H1[i + 1] * x1 + (s[i] - 'a') + 1) % mod1;
		H2[i] = (H2[i + 1] * x2 + (s[i] - 'a') + 1) % mod2;
	}
	vector<int> val1(n);
	for (int i = 0; i < n; i++) {
		int R = min(i + 1, n - i) + 1;
		int L = 0;
		while (R - L > 1) {
			int M = (L + R) / 2;
			int l1 = i - M + 1;
			int r1 = i + M - 1;
			if (check(l1, r1, h1, h2, H1, H2))
				L = M;
			else
				R = M;
		}
		val1[i] = L;
	}
	ll ans = 0;
	vector<pair<int, int> > even;
	int pt = 0;

	del_all();
	for (int i = 1; i < n; i += 2) {
		int len = val1[i];
		int t = max(0, i - 2 * len + 1);
		even.push_back({ t, i });
	}
	sort(even.begin(), even.end());
	
	for (int i = 0; i < n; i += 2) {
		while (pt < even.size()) {
			if (even[pt].first <= i) {
				add(even[pt].second, 1);
				pt++;
			} else
				break;
		}
		int len = val1[i];
		int L = i + 1;
		int R = min(MAXN - 1, i + 2 * len - 1);
		ans += getS(L, R);
	}

	del_all();
	even.clear();
	pt = 0;
	for (int i = 0; i < n; i += 2) {
		int len = val1[i];
		int t = max(0, i - 2 * len + 1);
		even.push_back({ t, i });
	}
	sort(even.begin(), even.end());

	for (int i = 1; i < n; i += 2) {
		while (pt < even.size()) {
			if (even[pt].first <= i) {
				add(even[pt].second, 1);
				pt++;
			} else
				break;
		}
		int len = val1[i];
		int L = i + 1;
		int R = min(MAXN - 1, i + 2 * len - 1);
		ans += getS(L, R);
	}

	//cout << ans << endl;

	del_all();
	even.clear();
	pt = 0;

	vector<int> val2(n);
	for (int i = 0; i < n; i++) {
		int L = 0;
		int R = min(i, n - i) + 1;
		while (R - L > 1) {
			int M = (L + R) / 2;
			int l1 = i - M;
			int r1 = i + M - 1;
			if (check(l1, r1, h1, h2, H1, H2))
				L = M;
			else
				R = M;
		}
		val2[i] = L;
	}
	//for (auto elem : val2)
	//	cout << elem << " ";
	//cout << endl;
	
	for (int i = 2; i < n; i += 2) {
		int len = val2[i];
		int t = max(0, i - 2 * len);
		even.push_back({ t, i });
	}
	sort(even.begin(), even.end());
	pt = 0;
	for (int i = 2; i < n; i += 2) {
		while (pt < even.size()) {
			if (even[pt].first <= i) {
				add(even[pt].second, 1);
				pt++;
			} else
				break;
		}
		//add(i, -1);
		int len = val2[i];
		int L = i + 1;
		int R = min(MAXN - 1, i + 2 * len);
		ans += getS(L, R);
	}
	del_all();
	even.clear();
	pt = 0;

	for (int i = 1; i < n; i += 2) {
		int len = val2[i];
		int t = max(0, i - 2 * len);
		even.push_back({ t, i });
	}
	sort(even.begin(), even.end());
	pt = 0;
	for (int i = 1; i < n; i += 2) {
		while (pt < even.size()) {
			if (even[pt].first <= i) {
				add(even[pt].second, 1);
				pt++;
			} else
				break;
		}
		//add(i, -1);
		int len = val2[i];
		int L = i + 1;
		int R = min(MAXN - 1, i + 2 * len);
		ans += getS(L, R);
	}

	cout << ans << endl;
}
//*/