#include <iostream>
#include <vector>
#include <set>
#include <algorithm>
#define int long long
using namespace std;

const int maxn = 500004;
const long long x = 29;
const long long mod = 1791791791;

long long hs[maxn], hsr[maxn];
long long xp[maxn];

int n;
string s;

int geths(int i, int j) {
    return ((hs[j] - hs[i] * xp[j - i]) % mod + mod) % mod;
}

int gethsr(int ti, int tj) {
    int i = n - tj;
    int j = n - ti;
    return ((hsr[j] - hsr[i] * xp[j - i]) % mod + mod) % mod;

}

int rsq1c[maxn * 4], rsq1n[maxn * 4], rsq2c[maxn * 4], rsq2n[maxn * 4];



int get1c(int t, int l, int r, int x, int y) {
    if (y <= l || r <= x)
        return 0;
    if (x <= l && r <= y)
        return rsq1c[t];
    int m = (l + r) / 2;
    return get1c(t * 2 + 1, l, m, x, y) + get1c(t * 2 + 2, m, r, x, y);
}

int get1n(int t, int l, int r, int x, int y) {
    if (y <= l || r <= x)
        return 0;
    if (x <= l && r <= y)
        return rsq1n[t];
    int m = (l + r) / 2;
    return get1n(t * 2 + 1, l, m, x, y) + get1n(t * 2 + 2, m, r, x, y);
}

int get2c(int t, int l, int r, int x, int y) {
    if (y <= l || r <= x)
        return 0;
    if (x <= l && r <= y)
        return rsq2c[t];
    int m = (l + r) / 2;
    return get2c(t * 2 + 1, l, m, x, y) + get2c(t * 2 + 2, m, r, x, y);
}

int get2n(int t, int l, int r, int x, int y) {
    if (y <= l || r <= x)
        return 0;
    if (x <= l && r <= y)
        return rsq2n[t];
    int m = (l + r) / 2;
    return get2n(t * 2 + 1, l, m, x, y) + get2n(t * 2 + 2, m, r, x, y);
}

void upd1c(int t, int l, int r, int x, int y, int delta) {
    if (y <= l || r <= x)
        return;
    if (x <= l && r <= y) {
        rsq1c[t] = delta;
        return;
    }
    int m = (l + r) / 2;
    upd1c(t * 2 + 1, l, m, x, y, delta);
    upd1c(t * 2 + 2, m, r, x, y, delta);
    rsq1c[t] = rsq1c[t * 2 + 1] + rsq1c[t * 2 + 2];
}

void upd1n(int t, int l, int r, int x, int y, int delta) {
    if (y <= l || r <= x)
        return;
    if (x <= l && r <= y) {
        rsq1n[t] = delta;
        return;
    }
    int m = (l + r) / 2;
    upd1n(t * 2 + 1, l, m, x, y, delta);
    upd1n(t * 2 + 2, m, r, x, y, delta);
    rsq1n[t] = rsq1n[t * 2 + 1] + rsq1n[t * 2 + 2];
}

void upd2c(int t, int l, int r, int x, int y, int delta) {
    if (y <= l || r <= x)
        return;
    if (x <= l && r <= y) {
        rsq2c[t] = delta;
        return;
    }
    int m = (l + r) / 2;
    upd2c(t * 2 + 1, l, m, x, y, delta);
    upd2c(t * 2 + 2, m, r, x, y, delta);
    rsq2c[t] = rsq2c[t * 2 + 1] + rsq2c[t * 2 + 2];
}

void upd2n(int t, int l, int r, int x, int y, int delta) {
    if (y <= l || r <= x)
        return;
    if (x <= l && r <= y) {
        rsq2n[t] = delta;
        return;
    }
    int m = (l + r) / 2;
    upd2n(t * 2 + 1, l, m, x, y, delta);
    upd2n(t * 2 + 2, m, r, x, y, delta);
    rsq2n[t] = rsq2n[t * 2 + 1] + rsq2n[t * 2 + 2];
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    cin >> n >> s;

    string sr = s;
    reverse(sr.begin(), sr.end());

    xp[0] = 1;
    for (int i = 1; i < maxn; ++i)
        xp[i] = (xp[i - 1] * x) % mod;

    for (int i = 1; i <= n; ++i) {
        hs[i] = (hs[i - 1] * x + s[i - 1]) % mod;
        hsr[i] = (hsr[i - 1] * x + sr[i - 1]) % mod;
    }

    vector<pair<int, int>> type1, type2;

    for (int i = 0; i < n; ++i) {

        int left = 0, right = i + 1, mid;

        while (right - left > 1) {
            mid = (left + right) / 2;
            int j = mid;
            if (i + j + 1 > n)
                right = mid;
            int t1 = gethsr(i - j, i + 1);
            int t2 = geths(i, i + j + 1);

            if (t1 == t2)
                left = mid;
            else
                right = mid;
        }
        type1.push_back({i, left});
    }

    for (int i = 0; i < n - 1; ++i) {
        if (s[i] != s[i + 1])
            continue;
        int left = 1, right = n, mid;

        while (right - left > 1) {
            mid = (left + right) / 2;
            int j = mid;
            if (i - j + 1 < 0 || i + 1 + j > n)
                right = mid;
            int t1 = gethsr(i - j + 1, i + 1);
            int t2 = geths(i + 1, i + 1 + j);
            if (t1 == t2)
                left = mid;
            else
                right = mid;
        }
        type2.push_back({i, left});
    }

    long long ans = 0;

    multiset<pair<int, int>> dead;

    for (int i = 0; i < type1.size(); ++i) {

        int npos = type1[i].first;
        int nd = type1[i].second;

        while (!dead.empty() && dead.begin()->first < npos) {
            int now = dead.begin()->second;
            dead.erase(dead.begin());
            upd1c(0, 0, n, now, now + 1, 0);
            upd1n(0, 0, n, now, now + 1, 0);
        }

        dead.insert({npos + 1 + 2 * nd, npos});

        if (npos % 2 == 0) {
            ans += get1n(0, 0, n, npos - 1 - 2 * nd, npos);
            upd1c(0, 0, n, npos, npos + 1, 1);
        }
        else {
            ans += get1c(0, 0, n, npos - 1 - 2 * nd, npos);
            upd1n(0, 0, n, npos, npos + 1, 1);
        }
    }

    dead.clear();

    for (int i = 0; i < type2.size(); ++i) {

        int npos = type2[i].first;
        int nd = type2[i].second;

        while (!dead.empty() && dead.begin()->first < npos) {
            int now = dead.begin()->second;
            dead.erase(dead.begin());
            upd2c(0, 0, n, now, now + 1, 0);
            upd2n(0, 0, n, now, now + 1, 0);
        }


        dead.insert({npos + 2 * nd, npos});

        if (npos % 2 == 0) {
            ans += get2c(0, 0, n, npos - 2 * nd, npos);
            upd2c(0, 0, n, npos, npos + 1, 1);
        }
        else {
            ans += get2n(0, 0, n, npos - 2 * nd, npos);
            upd2n(0, 0, n, npos, npos + 1, 1);
        }
    }

    cout << ans << '\n';

    return 0;
}
