#define PYHAEM_S_NOGI 0

#if PYHAEM_S_NOGI >= 1
#pragma GCC optimize("Ofast")
#pragma GCC target("tune=native,sse,sse2,sse3,avx,mmx")
#pragma GCC optimize("unroll-loops,no-stack-protector,btr-bb-exclusive,fast-math")
#endif

#include <bits/stdc++.h>

using namespace std;

#define int long long

#if PYHAEM_S_NOGI >= 2
const int BUFFER_SIZE = 65536;
char read_buffer[BUFFER_SIZE];
char* read_buffer_ptr = read_buffer;
char* read_buffer_end = read_buffer;
int getc_buffered() {
	if(read_buffer_ptr == read_buffer_end) {
		read_buffer_ptr = read_buffer;
		read_buffer_end = read_buffer + fread(read_buffer, 1, BUFFER_SIZE, stdin);
	 	if(read_buffer_ptr == read_buffer_end) {
			return -1;
		}
	}
	return *read_buffer_ptr++;
}
int readint() {
	int n = 0;
	int sign = 1;
	char c;
	do {
		c = getc_buffered();
	} while(!(('0' <= c && c <= '9') || c == '-'));
	if(c == '-') {
		sign = -1;
		c = getc_buffered();
	}
	do {
		n = n * 10 + (c - '0');
		c = getc_buffered();
	} while('0' <= c && c <= '9');
	return n;
}
#else
int readint() {
	int n;
	cin >> n;
	return n;
}
#endif

#if PYHAEM_S_NOGI >= 3
const int MAX_MEMORY = 300 * 1024 * 1024;
char MEMORY[MAX_MEMORY];
char* memory_ptr = MEMORY;

void* operator new(size_t sz) {
	void* res = memory_ptr;
	memory_ptr += sz;
	assert(memory_ptr < MEMORY + MAX_MEMORY);
	return res;
}

void operator delete(void*) {}
#endif

#if PYHAEM_S_NOGI >= 4
class DynamicArray {
public:
	int sz;
	int cap;
	int* ptr;

	DynamicArray() {
		sz = 0;
		cap = 16;
		ptr = new int[cap];
	}

	void push_back(int x) {
		if(sz == cap) {
			cap *= 2;
			int* new_ptr = new int[cap];
			memmove(new_ptr, ptr, sz * sizeof(int));
			delete[] ptr;
			ptr = new_ptr;
		}
		ptr[sz++] = x;
	}

	int size() {
		return sz;
	}

	int& operator[] (int i) {
		return ptr[i];
	}
};
#else
typedef vector<int> DynamicArray;
#endif

class SegmentTree {
public:
	vector<int> a;
	int n;

	SegmentTree(int sz) {
		n = sz;
		a.resize(4 * n);
	}

	int get(int v, int vl, int vr, int l, int r) {
		if(vl == l && vr == r) {
			return a[v];
		}
		int vm = (vl + vr) / 2;
		if(r <= vm) {
			return get(v * 2, vl, vm, l, r);
		} else if(l >= vm) {
			return get(v * 2 + 1, vm, vr, l, r);
		} else {
			return get(v * 2, vl, vm, l, vm) + get(v * 2 + 1, vm, vr, vm, r);
		}
	}
	int get(int l, int r) {
		return get(1, 0, n, l, r);
	}
	int get(int i) {
		return get(i, i + 1);
	}

	void set(int v, int vl, int vr, int i, int x) {
		if(vr - vl == 1) {
			a[v] = x;
			return;
		}
		int vm = (vl + vr) / 2;
		if(i < vm) {
			set(v * 2, vl, vm, i, x);
		} else {
			set(v * 2 + 1, vm, vr, i, x);
		}
	}
	void set(int i, int x) {
		set(1, 0, n, i, x);
	}
};

class Vertex {
public:
	int y, sz, value;
	Vertex* left;
	Vertex* right;
	Vertex(int x) {
		y = rand() ^ (rand() << 16);
		sz = 1;
		value = x;
		left = nullptr;
		right = nullptr;
	}
	void pull() {
		sz = (left ? left->sz : 0) + 1 + (right ? right->sz : 0);
	}
};

class Treap {
public:
	Vertex* root;
	Treap() {
		root = nullptr;
	}

	Vertex* merge(Vertex* a, Vertex* b) {
		if(a == nullptr) {
			return b;
		}
		if(b == nullptr) {
			return a;
		}
		if(a->y > b->y) {
			a->right = merge(a->right, b);
			a->pull();
			return a;
		} else {
			b->left = merge(a, b->left);
			b->pull();
			return b;
		}
	}

	pair<Vertex*, Vertex*> split(Vertex* v, int pos) {
		if(v == nullptr) {
			return make_pair(nullptr, nullptr);
		}
		if(pos >= (v->left ? v->left->sz : 0) + 1) {
			auto p = split(v->right, pos - (v->left ? v->left->sz : 0) - 1);
			v->right = p.first;
			v->pull();
			return make_pair(v, p.second);
		} else {
			auto p = split(v->left, pos);
			v->left = p.second;
			v->pull();
			return make_pair(p.first, v);
		}
	}
};

vector<int> z_function(string s) {
	int n = s.size();
	vector<int> z(n);
	int l = 0;
	int r = 0;
	for(int i = 1; i < n; i++) {
		if(i < r) {
			z[i] = min(r - i, z[i - l]);
		}
		while(i + z[i] < n && s[i + z[i]] == s[z[i]]) {
			z[i]++;
		}
		if(i + z[i] > r) {
			l = i;
			r = i + z[i];
		}
	}
	return z;
}

vector<int> pref_function(string s) {
	int n = s.size();
	vector<int> pref(n);
	for(int i = 1; i < n; i++) {
		pref[i] = pref[i - 1];
		while(pref[i] >= 1 && s[pref[i]] != s[i]) {
			pref[i] = pref[pref[i] - 1];
		}
		if(s[pref[i]] == s[i]) {
			pref[i]++;
		}
	}
	return pref;
}

const int MOD = 1000000009LL;
const int POW = 5757;

vector<int> hsh, hshrev, pows;
int n;

bool is(int l, int r) {
	return ((hsh[r - 1] - (l == 0 ? 0 : hsh[l - 1] * pows[r - l])) - (hshrev[l] - (r == n ? 0 : hshrev[r] * pows[r - l]))) % MOD == 0;
}

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	cin >> n;
	string s;
	cin >> s;

	hsh.resize(n);
	hshrev.resize(n + 1);
	for(int i = 0; i < n; i++) {
		hsh[i] = ((i == 0 ? 0 : hsh[i - 1]) * POW + s[i]) % MOD;
	}
	for(int i = n - 1; i >= 0; i--) {
		hshrev[i] = (hshrev[i + 1] * POW + s[i]) % MOD;
	}
	pows.resize(n + 1);
	pows[0] = 1;
	for(int i = 1; i <= n; i++) {
		pows[i] = pows[i - 1] * POW % MOD;
	}

	int cnt = 0;
	for(int mid = 0; mid < n; mid++) {
		for(int len = 1; mid - len >= 0 && mid + len <= n; len++) {
			bool a = (
				hsh[mid - 1]
				- hshrev[mid - len]
				+ (
					hshrev[mid]
					- (mid == len ? 0 : hsh[mid - len - 1])
				) * pows[len]
			) % MOD == 0;
			bool b = (
				hsh[mid + len - 1]
				- hshrev[mid]
				+ (
					hshrev[mid + len]
					- (mid == 0 ? 0 : hsh[mid - 1])
				) * pows[len]
			) % MOD == 0;
			if(a && b) {
				cnt++;
			}
		}
	}
	cout << cnt << endl;

#if PYHAEM_S_NOGI >= 3
	cerr << endl << endl << "Memory used: " << (double)((memory_ptr - MEMORY) * 10000 / MAX_MEMORY) / 100 << "%" << endl;
#endif

	return 0;
}