#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <string>

using ll = long long;
using namespace std;

//#define int long long
#define rep(i, a, b) for (int i = (a); i < (b); ++i)
#define per(i, a, b) for (int i = (a) - 1; i >= (b); --i)
#define all(a) a.begin(), a.end()
#define pb push_back
#define mp make_pair

struct strHash {
    int n;
    ll b, MOD;
    vector<ll> p, norm, rev;
    strHash(string& s, ll MOD, ll b) : MOD(MOD), b(b) {
        n = s.size();
        norm.resize(n + 1);
        rep(i, 1, n + 1) norm[i] = (norm[i - 1] * b + s[i - 1]) % MOD;
        rev.resize(n + 1);
        per(i, n, 0) rev[i] = (rev[i + 1] * b + s[i]) % MOD;
        p.resize(n + 1);
        p[0] = 1;
        rep(i, 1, n + 1) p[i] = (p[i - 1] * b) % MOD;
    }
    ll getNorm(int L, int R) {
        if (R < L) swap(L, R);
        R++;
        ll ans = norm[R] - norm[L] * p[R - L] % MOD;
        while (ans < 0) ans += MOD;
        return ans;
    }
    ll getRev(int L, int R) {
        if (R < L) swap(L, R);
        R++;
        ll ans = rev[L] - rev[R] * p[R - L] % MOD;
        while (ans < 0) ans += MOD;
        return ans;
    }
};

struct Tree {
    vector<int> tree;
    Tree(int n) {
        tree.resize(4 * n);
    }
    void update(int v, int vl, int vr, int pos) {
        if (pos < vl || vr <= pos) return;
        if (vr - vl == 1) {
            tree[v]++;
            return;
        }
        int vm = (vl + vr) / 2;
        update(2 * v, vl, vm, pos);
        update(2 * v + 1, vm, vr, pos);
        tree[v] = tree[2 * v] + tree[2 * v + 1];
    }
    int get(int v, int vl, int vr, int l, int r) {
        if (r <= vl || vr <= l) return 0;
        if (l <= vl && vr <= r) return tree[v];
        int vm = (vl + vr) / 2;
        return get(2 * v, vl, vm, l, r) + get(2 * v + 1, vm, vr, l, r);
    }
};

bool isPal(int L, int R, strHash& h) {
    return h.getNorm(L, R) == h.getRev(L, R);
}

ll even(string& s, int n, strHash& h) {
    vector<int> mx(n, -1);
    rep(i, 1, n - 1) {
        int L = 0, R = n + 1;
        while (R - L > 1) {
            int mid = (L + R) / 2;
            int left = i - mid, right = i + mid - 1;
            if (left < 0 || right >= n) {
                R = mid;
                continue;
            }
            if (isPal(left, right, h)) L = mid;
            else R = mid;
        }
        mx[i] = L;
    }
    ll ans = 0;
    Tree tr1 = Tree(n);
    Tree tr2 = Tree(n);
    rep(i, 0, n) {
        if (mx[i] == -1) continue;
        if (i % 2 == 0) {
            ans += tr2.get(1, 0, n, i - mx[i] - 1, n);
            tr1.update(1, 0, n, i + mx[i] - 1);
        } else {
            ans += tr1.get(1, 0, n, i - mx[i] - 1, n);
            tr2.update(1, 0, n, i + mx[i] - 1);
        }
    }
    return ans;
}

ll odd(string& s, int n, strHash& h) {
    vector<int> mx(n, 0);
    rep(i, 1, n - 1) {
        int L = 0, R = n + 1;
        while (R - L > 1) {
            int mid = (L + R) / 2;
            int left = i - mid, right = i + mid;
            if (left < 0 || right >= n) {
                R = mid;
                continue;
            }
            if (isPal(left, right, h)) L = mid;
            else R = mid;
        }
        mx[i] = L;
    }
    ll ans = 0;
    Tree tr1 = Tree(n);
    Tree tr2 = Tree(n);
    rep(i, 0, n) {
        if (i % 2 == 0) {
            ans += tr2.get(1, 0, n, i - mx[i] - 1, n);
            tr1.update(1, 0, n, i + mx[i]);
        } else {
            ans += tr1.get(1, 0, n, i - mx[i] - 1, n);
            tr2.update(1, 0, n, i + mx[i]);
        }
    }
    return ans;
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(0), cout.tie(0);
    int n;
    cin >> n;
    string s;
    cin >> s;
    strHash h = strHash(s, 2286661337, 239);
    cout << even(s, n, h) + odd(s, n, h) << '\n';
}
