#include <algorithm>
#include <iostream>
#include <vector>
#include <string>

using namespace std;

typedef int64_t ll;

const ll MOD1 = 2286661337;
const ll MOD2 = 1'000'000'009;

struct Hash {
	ll n;
	string s;
	vector<ll> hs1;
	vector<ll> hs2;
	vector<ll> d1;
	vector<ll> d2;

	Hash() {}

	Hash(string s) 
		: s(s) {
		n = (ll)s.size();
		hs1.resize(n);
		d1.resize(n);
		hs2.resize(n);
		d2.resize(n);
		d2[0] = d1[0] = 1;
		hs2[0] = hs1[0] = s[0] - 'a' + 1;
		for (ll i = 1; i < n; ++i) {
			d1[i] = d1[i - 1] * 47 % MOD1;
			hs1[i] = (hs1[i - 1] + (s[i] - 'a' + 1) * d1[i]) % MOD1;
			d2[i] = d2[i - 1] * 179 % MOD2;
			hs2[i] = (hs2[i - 1] + (s[i] - 'a' + 1) * d2[i]) % MOD2;
		}
	}

	inline pair<ll, ll> get(ll left, ll right) {
		if (left == 0) {
			return make_pair(hs1[right] * d1[n - 1] % MOD1, hs2[right] * d2[n - 1] % MOD2);
		}
		return make_pair((hs1[right] - hs1[left - 1] + MOD1) * d1[n - 1 - left] % MOD1, (hs2[right] - hs2[left - 1] + MOD2) * d2[n - 1 - left] % MOD2);
	}
};

ll n;
Hash hs;
Hash rhs;

const ll BUBEN = 20;

bool is_pal(ll left, ll right) {
	if (hs.get(left, right) != rhs.get(n - right - 1, n - left - 1)) {
		return false;
	}
	for (ll i = 0; i < BUBEN; ++i) {
		ll id = rand() % (right - left + 1);
		if (hs.get(left, left + id) != rhs.get(n - right - 1, n - right + id - 1)) {
			return false;
		}
	}
	return true;
}

int main() {
	cin >> n;
	string s;
	cin >> s;
	string kek;
	for (ll i = 0; i < 26; ++i) {
		kek.push_back('a' + i);
	}
	random_shuffle(kek.begin(), kek.end());
	for (ll i = 0; i < n; ++i) {
		s[i] = kek[s[i] - 'a'];
	}
	hs = Hash(s);
	reverse(s.begin(), s.end());
	rhs = Hash(s);
	ll result = 0;
	for (ll len = 1; len * 2 <= n; ++len) {
		for (ll i = 0; i + 2 * len - 1 < n; ++i) {
			if (!is_pal(i, i + len - 1)) {
				continue;
			}
			if (!is_pal(i + len, i + 2 * len - 1)) {
				continue;
			}
			++result;
		}
	}
	cout << result << endl;
	return 0;
}
