#include <iostream>
#include <random>
#include <algorithm>

using namespace std;

template <typename T>
T *arr(int n) {
    return (T*) malloc(n * sizeof(T));
}

template <typename T>
T *arr(int n, T val) {
    auto res = arr<T>(n);
    fill(res, res + n, val);
    return res;
}

struct Graph {
    int *head, *nxt, *eto, ptr = 0;
    Graph(int n, int m) {
        head = arr<int>(n, -1);
        nxt = arr<int>(m);
        eto = arr<int>(m);
    }
    void add_edge(int from, int to) {
        eto[ptr++] = to;
        nxt[ptr - 1] = head[from];
        head[from] = ptr - 1;
    }
};

void run();

signed main() {
    ios::sync_with_stdio(false);
    cout.tie(0);
    run();
}

bool find_path(int v, int fin, Graph &tree, vector<int> &vis, int p = -1) {
    vis.push_back(v);
    if (v == fin)
        return true;
    for (int e = tree.head[v]; e != -1; e = tree.nxt[e]) {
        int u = tree.eto[e];
        if (u == p)
            continue;
        if (find_path(u, fin, tree, vis, v))
            return true;
    }
    vis.pop_back();
    return false;
}

bool dfs(int v, Graph &gr, int *ans, int *clr) {
    ans[v] = 1;
    clr[v] = 1;
    for (int e = gr.head[v]; e != -1; e = gr.nxt[e]) {
        int u = gr.eto[e];
        if (clr[u] == 1) {
            return true;
        }
        if (clr[u] == 0) {
            if (dfs(u, gr, ans, clr))
                return true;
        }
        ans[v] = max(ans[v], ans[u] + 1);
    }
    clr[v] = 2;
    return false;
}

void run() {
    int n, m, t;
    cin >> n >> m >> t;
    Graph tree(n + 1, n * 2);
    for (int i = 0; i < n - 1; i++) {
        int u, v;
        cin >> u >> v;
        tree.add_edge(u, v);
        tree.add_edge(v, u);
    }
    vector<pair<int, int>> new_edges;
    for (int i = 0; i < m; i++) {
        int a, b;
        cin >> a >> b;
        vector<int> path;
        find_path(a, b, tree, path);
        for (int j = 1; j < (int) path.size(); j++) {
            new_edges.emplace_back(path[j], path[j - 1]);
        }
    }
    Graph gr(n + 1, new_edges.size());
    for (auto p: new_edges) {
        gr.add_edge(p.first, p.second);
    }
    auto ans = arr<int>(n + 1, 0);
    auto clr = arr<int>(n + 1, 0);
    for (int v = 1; v <= n; v++) {
        if (clr[v] == 2)
            continue;
        if (dfs(v, gr, ans, clr)) {
            cout << -1;
            return;
        }
    }
    int k = *max_element(ans, ans + n + 1);
    cout << k << endl;
    for (int i = 1; i <= n; i++) {
        cout << ans[i] << ' ';
    }
}