#include <iostream>
#include <cstdint>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <algorithm>
#include <string>
#include <vector>

struct Path {
    int32_t begin;
    int32_t end;
    int32_t lca;
    int32_t length;

    Path(int32_t begin_, int32_t end_, int32_t lca_, int32_t length_) {
        begin = begin_;
        end = end_;
        lca = lca_;
        length = length_;
    }

    bool operator <(Path& other) {
        return length > other.length;
    }
};

void dfs(int32_t vertex, std::vector<int32_t>* tree, bool* visited, std::vector<int32_t>& euler, int32_t* heights, int32_t height,
        int32_t parent, int32_t* parents) {

    visited[vertex] = true;
    heights[vertex] = height;
    parents[vertex] = parent;
    euler.push_back(vertex);
    for(int32_t i = 0; i < tree[vertex].size(); i++)
        if(!visited[tree[vertex][i]]) {
            dfs(tree[vertex][i], tree, visited, euler, heights, height + 1, vertex, parents);
            euler.push_back(vertex);
        }
}

int32_t get_min_index(int32_t left, int32_t right, int32_t** sparse_table) {
    if(right < left) {
        int32_t tmp = left;
        left = right;
        right = tmp;
    }
    int32_t length = right - left + 1;

    int32_t level = 0;
    while ((1 << level) <= length)
        level++;
    level--;

    return std::min(sparse_table[level][left], sparse_table[level][right - (1 << level) + 1]);
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(0);
    std::cout.tie(0);
    
    int32_t n, num_paths, t;
    std::cin >> n >> num_paths >> t;

    std::vector<int32_t>* tree = new std::vector<int32_t>[n];
    for(int32_t i = 0 ;i < n - 1; i++) {
        int32_t  src, dst;
        std::cin >> src >> dst;
        src--;
        dst--;
        tree[src].push_back(dst);
        tree[dst].push_back(src);
    }

    std::pair<int32_t, int32_t>* paths = new std::pair<int32_t, int32_t>[num_paths];
    for(int32_t i = 0 ; i < num_paths; i++) {
        std::cin >> paths[i].first >> paths[i].second;
        paths[i].first--;
        paths[i].second--;
    }

    std::vector<int32_t> euler;
    bool* visited = new bool[n];
    int32_t* heights = new int32_t[n], *parents = new int32_t[n];
    for(int32_t i = 0 ;i < n; i++)
        visited[i] = false;

    dfs(0, tree, visited, euler, heights, 0, -1, parents);

    std::map<int32_t, int32_t> first_euler;
    for(int32_t i = 0; i < euler.size(); i++) {
        if(first_euler.find(euler[i]) == first_euler.end())
            first_euler[euler[i]] = i;
    }

    int32_t max_level = 0;
    while ((1 << max_level) <= euler.size())
        max_level++;
    max_level--;

    int32_t** sparse_table = new int32_t*[max_level + 1];
    for(int32_t i = 0; i <= max_level; i++) {
        sparse_table[i] = new int32_t[euler.size() - (1 << i) + 1];
        for(int32_t j = 0 ; j < euler.size() - (1 << i) + 1; j++) {
            if(i == 0)
                sparse_table[i][j] = j;
            else {
                int32_t vertex_left = euler[sparse_table[i - 1][j]];
                int32_t vertex_right = euler[sparse_table[i - 1][j + (1 << i) / 2]];
                if(heights[vertex_left] < heights[vertex_right])
                    sparse_table[i][j] = sparse_table[i - 1][j];
                else
                    sparse_table[i][j] = sparse_table[i - 1][j + (1 << i) / 2];
            }
        }
    }

    int32_t* answer = new int32_t[n];
    for(int32_t i = 0 ; i < n; i++)
        answer[i] = 1;

    bool* marked = new bool[n];
    for(int32_t i = 0; i < n; i++)
        marked[i] = false;

    std::vector<Path> paths2;
    for(int32_t i = 0 ; i < num_paths; i++) {
        int32_t lca_index = get_min_index(first_euler[paths[i].first], first_euler[paths[i].second], sparse_table);
        int32_t lca_vertex = euler[lca_index];
        paths2.emplace_back(paths[i].first, paths[i].second, lca_vertex, heights[paths[i].first] + heights[paths[i].second] - 2 * heights[lca_vertex]);
    }
    std::sort(paths2.begin(), paths2.end());

    for(int32_t i = 0 ; i < num_paths; i++) {
        if(!marked[paths2[i].begin] && !marked[paths2[i].end]) {
            int32_t cur = paths2[i].begin;
            int32_t step = 1;
            while(cur != parents[paths2[i].lca]) {
                answer[cur] = step;
                marked[cur] = true;
                step++;
                cur = parents[cur];
            }
            cur = paths2[i].end;
            step = paths2[i].length + 1;
            while(cur != parents[paths2[i].lca]) {
                answer[cur] = step;
                marked[cur] = true;
                step--;
                cur = parents[cur];
            }
        } else if(marked[paths2[i].begin]) {
            int32_t first_num = answer[paths2[i].begin] - paths2[i].length;
            int32_t cur = paths2[i].end;
            int32_t step = first_num;
            while(cur != parents[paths2[i].lca]) {
                answer[cur] = step;
                marked[cur] = true;
                step++;
                cur = parents[cur];
            }
        } else {
            int32_t first_num = answer[paths2[i].end] - paths2[i].length;
            int32_t cur = paths2[i].begin;
            int32_t step = first_num;
            while(cur != parents[paths2[i].lca]) {
                answer[cur] = step;
                marked[cur] = true;
                step++;
                cur = parents[cur];
            }
        }
    }
    int32_t max_answer = answer[0];
    for(int32_t i = 0 ; i < n; i++)
        max_answer = std::max(max_answer, answer[i]);
    std::cout << max_answer << "\n";

    for(int32_t i = 0 ; i < n; i++)
        std::cout << answer[i] << " ";
    return 0;
}