#include <algorithm>
#include <iostream>
#include <vector>
#include <string>
#include <map>

using namespace std;

typedef int ll;

map<pair<ll, ll>, ll> edges_id;
vector<pair<ll, ll>> edges;
vector<vector<ll>> graph;
vector<vector<ll>> tree;
vector<vector<ll>> dp;
vector<ll> result;
vector<ll> color;
vector<ll> tout;
vector<ll> tin;
vector<ll> sum;
ll timer = 0;

void dfs(ll v, ll p) {
	dp[0][v] = p;
	tin[v] = ++timer;
	for (ll u : tree[v]) {
		if (u == p) {
			continue;
		}
		dfs(u, v);
	}
	tout[v] = ++timer;
}


bool is_parent(ll parent, ll child) {
	return tin[parent] <= tin[child] && tout[child] <= tout[parent];
}

ll get_pre_lca(ll v, ll u) {
	for (ll i = 19; i >= 0; --i) {
		if (!is_parent(dp[i][v], u)) {
			v = dp[i][v];
		}
	}
	return v;
}

ll get_lca(ll v, ll u) {
	v = get_pre_lca(v, u);
	if (!is_parent(v, u)) {
		return dp[0][v];
	}
	return v;
}

void dfs_sum(ll v, ll p) {
	for (ll u : tree[v]) {
		if (u == p) {
			continue;
		}
		dfs_sum(u, v);
		sum[v] += sum[u];
	}
}

bool dfs_check(ll v, ll c) {
	color[v] = c;
	for (ll u : graph[v]) {
		if (color[u] == c) {
			return true;
		}
		if (color[u]) {
			continue;
		}
		if (dfs_check(u, 3 - c)) {
			return true;
		}
	}
	return false;
}

void dfs_edges(ll v) {
	color[v] = timer;
	for (ll u : graph[v]) {
		if (color[u] == timer) {
			continue;
		}
		if (edges[u].second == edges[v].second) {
			swap(edges[u].first, edges[u].second);
		}
		if (edges[u].first == edges[v].first) {
			swap(edges[u].first, edges[u].second);
		}
		dfs_edges(u);
	}
}

void dfs_result(ll v) {
	result[v] = 1;
	for (ll u : tree[v]) {
		if (color[edges_id[make_pair(v, u)]] == -1) {
			continue;
		}
		if (result[u] == 0) {
			dfs_result(u);
		}
		result[v] = max(result[v], result[u] + 1);
	}
}

int main() {
	ios_base::sync_with_stdio(false);
	cout.tie(nullptr);
	cin.tie(nullptr);

	ll n, m, t;
	cin >> n >> m >> t;
	graph.resize(n - 1);
	color.resize(n);
	tree.resize(n);
	tout.resize(n);
	tin.resize(n);
	sum.resize(n);
	dp.resize(20, vector<ll>(n));
	for (ll i = 0; i < n - 1; ++i) {
		ll v, u;
		cin >> v >> u;
		--v;
		--u;
		tree[v].emplace_back(u);
		tree[u].emplace_back(v);
		edges_id[make_pair(v, u)] = i;
		edges_id[make_pair(u, v)] = i;
		edges.emplace_back(v, u);
	}
	dfs(0, 0);
	for (ll i = 1; i < 20; ++i) {
		for (ll v = 0; v < n; ++v) {
			dp[i][v] = dp[i - 1][dp[i - 1][v]];
		}
	}
	vector<pair<ll, ll>> path;
	for (ll i = 0; i < m; ++i) {
		ll v, u;
		cin >> v >> u;
		--v;
		--u;
		path.emplace_back(v, u);
		ll v0 = get_pre_lca(v, u);
		ll u0 = get_pre_lca(u, v);
		++sum[v];
		--sum[v0];
		++sum[u];
		--sum[u0];
		if (is_parent(v, u) || is_parent(u, v)) {
			continue;
		}
		ll lca = dp[0][v0];
		ll id_1 = edges_id[make_pair(lca, v0)];
		ll id_2 = edges_id[make_pair(lca, u0)];
		graph[id_1].emplace_back(id_2);
		graph[id_2].emplace_back(id_1);
	}
	dfs_sum(0, 0);
	for (ll v = 0; v < n; ++v) {
		if (sum[v]) {
			ll p = dp[0][v];
			ll pp = dp[0][p];
			ll id_1 = edges_id[make_pair(p, v)];
			ll id_2 = edges_id[make_pair(pp, p)];
			graph[id_1].emplace_back(id_2);
			graph[id_2].emplace_back(id_1);
		}
	}

	for (ll v = 0; v < n - 1; ++v) {
		if (color[v]) {
			continue;
		}
		if (dfs_check(v, 1)) {
			cout << -1 << endl;
			return 0;
		}
	}
	for (ll& i : color) {
		color[i] = -1;
	}
	timer = -1;
	for (ll v = 0; v < n - 1; ++v) {
		if (color[v] != -1) {
			continue;
		}
		++timer;
		dfs_edges(v);
	}

	for (ll& i : sum) {
		i = 0;
	}

	for (auto i : path) {
		ll v = i.first;
		ll u = i.second;
		ll lca = get_lca(v, u);
		sum[v] += 1;
		sum[u] += 1;
		sum[lca] -= 2;
	}

	dfs_sum(0, 0);

	for (ll v = 1; v < n; ++v) {
		if (sum[v]) {
			continue;
		}
		ll u = dp[0][v];
		color[edges_id[make_pair(v, u)]] = -1;
	}

	tree.clear();
	tree.resize(n);
	for (auto e : edges) {
		tree[e.first].emplace_back(e.second);
	}
	result.resize(n);
	for (ll v = 0; v < n; ++v) {
		if (result[v]) {
			continue;
		}
		dfs_result(v);
	}
	ll mx = 0;
	for (ll i : result) {
		mx = max(i, mx);
	}
	cout << mx << endl;
	for (ll i : result) {
		cout << i << " ";
	}
	cout << endl;
	return 0;
}
