#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <cstring>

using namespace std;

struct edge{
    int v, u;
    bool dir = 0, good = 0;
    edge() {}
    edge(int v, int u): v(v), u(u) {}
    int get(int x) {
        if (x == v) return u;
        return v;
    }
};

const int N = 5e5 + 7;
set <int> rest;
vector <int> g[N], sorted;
vector <pair <int, int>> has[N];
int color[N], best[N];
bool vis[N];
vector <edge> e;
vector <vector <int>> paths;
int tin[N], tout[N], par[N];
int T = 0;
int best_res = N;

void dfs(int v, int p = -1) {
    par[v] = p;
    tin[v] = T++;
    for (int u : g[v]) {
        if (u != p) dfs(e[u].get(v), u);
    }
    tout[v] = T - 1;
}

void dfs2(int v) {
    if (vis[v]) return;
    vis[v] = 1;
    for (int u : g[v]) {
        if (e[u].dir && e[u].v == v) dfs2(e[u].u);
    }
    sorted.push_back(v);
}

bool is_ancestor(int v, int u) {
    return (tin[v] <= tin[u] && tout[u] <= tout[v]);
}

void to_path(int v, int u) {
    vector <int> fi, se;
    while (!is_ancestor(v, u)) {
        fi.push_back(par[v]);
        v = e[par[v]].get(v);
    }
    while (u != v) {
        se.push_back(par[u]);
        u = e[par[u]].get(u);
    }
    reverse(se.begin(), se.end());
    for (int elem : se) fi.push_back(elem);
    for (int i = 0; i < (int) fi.size(); i++) {
        has[fi[i]].push_back({paths.size(), i});
    }
    paths.push_back(fi);
}

void relax(int n) {
    memset(vis, 0, sizeof(vis));
    memset(color, 0, sizeof(color));
    sorted.clear();
    for (int i = 1; i <= n; i++) {
        if (!vis[i]) dfs2(i);
    }
    int maxi = -1;
    if (sorted.empty()) exit(0);
    for (int elem : sorted) {
        color[elem] = 1;
        for (int u : g[elem]) {
            if (e[u].dir && e[u].v == elem) {
                color[elem] = max(color[elem], color[e[u].u] + 1);
            }
        }
        maxi = max(maxi, color[elem]);
    }
    if (maxi < best_res) {
        best_res = maxi;
        memcpy(best, color, sizeof(color));
    }
}


int main() {
    //freopen("input.txt", "r", stdin);
    ios_base :: sync_with_stdio(0);
    cin.tie(0);
    int n, m, t;
    cin >> n >> m >> t;
    for (int i = 0; i < n - 1; i++) {
        int v, u;
        cin >> v >> u;
        e.push_back(edge(v, u));
        g[v].push_back(i);
        g[u].push_back(i);
    }
    dfs(1);
    for (int i = 0; i < m; i++) {
        int v, u;
        cin >> v >> u;
        to_path(v, u);
        rest.insert(i);
    }
    vector <int> cur;
    cur.push_back(0);
    set <int> noww;
    vector <vector <int>> comps;
    memset(color, 0, sizeof(color));
    while (1) {
        if (cur.empty()) {
            vector <int> curr;
            for (int elem : noww) curr.push_back(elem);
            noww.clear();
            if (!curr.empty()) comps.push_back(curr);
            if (rest.empty()) break;
            cur.push_back(*rest.begin());
            rest.erase(rest.begin());
        }
        int pos = cur.back();
        cur.pop_back();
        int start = 0;
        vector <int> now = paths[pos];
        for (int elem : now) noww.insert(elem);
        for (int i = 0; i < (int) now.size(); i++) {
            if (e[now[i]].dir) {
                start = i;
                break;
            }
        }
        if (!e[now[start]].dir) {
            e[now[start]].dir = 1;
            for (auto elem : has[now[start]]) {
                if (rest.find(elem.first) != rest.end()) {
                    rest.erase(elem.first);
                    cur.push_back(elem.first);
                }
            }
        }
        for (int i = start + 1; i < (int) now.size(); i++) {
            int nv;
            if (e[now[i]].v == e[now[i - 1]].u || e[now[i]].u == e[now[i - 1]].v) nv = e[now[i]].v;
            else nv = e[now[i]].u;
            if (e[now[i]].dir && e[now[i]].v != nv) {
                cout << -1 << endl;
                exit(0);
            }
            if (e[now[i]].v != nv) swap(e[now[i]].v, e[now[i]].u);
            if (!e[now[i]].dir) {
                e[now[i]].dir = 1;
                for (auto elem : has[now[i]]) {
                    if (rest.find(elem.first) != rest.end()) {
                        rest.erase(elem.first);
                        cur.push_back(elem.first);
                    }
                }
            }
        }
        for (int i = start - 1; i >= 0; i--) {
            int nv;
            if (e[now[i]].v == e[now[i + 1]].u || e[now[i]].u == e[now[i + 1]].v) nv = e[now[i]].v;
            else nv = e[now[i]].u;
            if (e[now[i]].dir && e[now[i]].v != nv) {
                cout << -1 << endl;
                exit(0);
            }
            if (e[now[i]].v != nv) swap(e[now[i]].v, e[now[i]].u);
            if (!e[now[i]].dir) {
                for (auto elem : has[now[i]]) {
                    if (rest.find(elem.first) != rest.end()) {
                        rest.erase(elem.first);
                        cur.push_back(elem.first);
                    }
                }
            }
            e[now[i]].dir = 1;
        }
    }
    if (m <= 15) {
        int sz = comps.size();
        for (int mask = 0; mask < (1 << sz); mask++) {
            for (int i = 0; i < sz; i++) {
                if (mask & (1 << i)) {
                    for (int elem : comps[i]) {
                        swap(e[comps[i][elem]].v, e[comps[i][elem]].u);
                    }
                }
            }
            relax(n);
            for (int i = 0; i < sz; i++) {
                if (mask & (1 << i)) {
                    for (int elem : comps[i]) {
                        swap(e[comps[i][elem]].v, e[comps[i][elem]].u);
                    }
                }
            }
        }
    } else {
        relax(n);
    }
    cout << best_res << endl;
    for (int i = 1; i <= n; i++) cout << best[i] << ' ';
    cout << endl;
    return 0;
}
