#include <iostream>
#include <random>
#include <algorithm>

using namespace std;

template <typename T>
T *arr(int n) {
    return (T*) malloc(n * sizeof(T));
}

template <typename T>
T *arr(int n, T val) {
    auto res = arr<T>(n);
    fill(res, res + n, val);
    return res;
}

struct Graph {
    int *head, *nxt, *eto, *efrom, ptr = 0;
    Graph(int n, int m) {
        head = arr<int>(n, -1);
        nxt = arr<int>(m);
        eto = arr<int>(m);
        efrom = arr<int>(m);
    }
    void add_edge(int from, int to) {
        eto[ptr++] = to;
        nxt[ptr - 1] = head[from];
        efrom[ptr - 1] = from;
        head[from] = ptr - 1;
    }
};

void run();

signed main() {
    ios::sync_with_stdio(false);
    cout.tie(0);
    run();
}

bool find_path(int v, int fin, Graph &tree, vector<int> &vis, int p = -1) {
    if (v == fin)
        return true;
    for (int e = tree.head[v]; e != -1; e = tree.nxt[e]) {
        int u = tree.eto[e];
        if (u == p)
            continue;
        if (find_path(u, fin, tree, vis, v)) {
            vis.push_back(e);
            return true;
        }
    }
    return false;
}

int get(int a, int *par) {
    if (a == par[a])
        return a;
    return par[a] = get(par[a], par);
}

void join(int u, int v, int *par, int *ran) {
    u = get(u, par);
    v = get(v, par);
    if (u == v)
        return;
    if (ran[u] < ran[v]) {
        par[u] = v;
    } else {
        par[v] = u;
        if (ran[u] == ran[v])
            ran[u]++;
    }
}

void dfs(int v, Graph &gr, int *ans, Graph &tree, int p = -1) {
    //cerr << tree.efrom[v] << ' ' << tree.eto[v] << endl;
    for (int e = gr.head[v]; e != -1; e = gr.nxt[e]) {
        int u = gr.eto[e];
        if (u == p)
            continue;
        dfs(u, gr, ans, tree, v);
    }
    ans[tree.efrom[v]] = max(ans[tree.efrom[v]], ans[tree.eto[v]] + 1);
}

void run() {
    int n, m, t;
    cin >> n >> m >> t;
    Graph tree(n + 1, n * 2);
    for (int i = 0; i < n - 1; i++) {
        int u, v;
        cin >> u >> v;
        tree.add_edge(u, v);
        tree.add_edge(v, u);
    }
    int k = n * 2 - 2;
    auto par = arr<int>(k);
    for (int i = 0; i < k; i++)
        par[i] = i;
    auto ran = arr<int>(k, 1);
    vector<pair<int, int>> new_edges;
    for (int i = 0; i < m; i++) {
        int a, b;
        cin >> a >> b;
        vector<int> path;
        find_path(a, b, tree, path);
        for (int j = 1; j < (int) path.size(); j++) {
            if (get(path[j - 1], par) == get(path[j], par))
                continue;
            join(path[j - 1], path[j], par, ran);
            //cerr << path[j - 1] << ' ' << path[j] << endl;
            join(path[j - 1] ^ 1, path[j] ^ 1, par, ran);
            //cerr << (path[j - 1] ^ 1) << ' ' << (path[j] ^ 1) << endl;
            new_edges.emplace_back(path[j - 1], path[j]);
            new_edges.emplace_back(path[j], path[j - 1]);
            new_edges.emplace_back(path[j - 1] ^ 1, path[j] ^ 1);
            new_edges.emplace_back(path[j] ^ 1, path[j - 1] ^ 1);
        }
    }
    //cerr << endl;
    for (int i = 0; i < n - 1; i++) {
        //cerr << get(i * 2, par) << ' ' << get(i * 2 + 1, par) << endl;
        if (get(i * 2, par) == get(i * 2 + 1, par)) {
            cout << -1;
            return;
        }
    }
    Graph gr(k, new_edges.size());
    for (auto p: new_edges) {
        gr.add_edge(p.first, p.second);
    }
    auto ans = arr<int>(n + 1, 1);
    auto taken = arr<bool>(k, false);
    for (int i = 0; i < k; i++) {
        if (taken[get(i ^ 1, par)])
            continue;
        taken[get(i, par)] = true;
        dfs(i, gr, ans, tree);
    }
    int p = *max_element(ans, ans + n + 1);
    cout << p << endl;
    for (int v = 1; v <= n; v++) {
        cout << ans[v] << ' ';
    }
}
