#include<iostream>
#include<vector>

using namespace std;

const int MAXN = 50 + 5;

struct Edge
{
    int u, v;
    int ind;

    Edge(){}
    Edge(int u, int v, int ind)
    {
        this->u = u;
        this->v = v;
        this->ind = ind;
    }
};

int n, m, T;
vector <Edge> edges;
pair <int, int> routes[MAXN];

vector <Edge> graph[MAXN];
vector <int> route2Edges[MAXN];

struct ConfigurationBuilder
{
    bool used[MAXN];
    vector <int> graph[MAXN];

    int cMax;
    int c[MAXN];

    void reset()
    {
        cMax = 0;
        topSort.clear();
        for(int i = 1;i<=n;i++)
        {
            c[i] = 0;
            used[i] = false;
            graph[i].clear();
        }
    }

    void addEdge(Edge e)
    {
        graph[e.u].push_back(e.v);
    }

    vector <int> topSort;
    void DFS(int x)
    {
        used[x] = true;
        for(int y: graph[x])
        {
            if(used[y]==true) continue;
            DFS(y);
        }

        topSort.push_back(x);
    }

    void calc()
    {
        for(int i = 1;i<=n;i++)
        {
            if(used[i]==false)
            {
                DFS(i);
            }
        }

        for(int x: topSort)
        {
            c[x] = 1;
            for(int y: graph[x])
            {
                c[x] = max(c[x], c[y]+1);
            }

            cMax = max(c[x], cMax);
        }
    }
};


bool DFS(int x, int last, int root, int destination)
{
    if(x==destination)
    {
        return true;
    }

    for(Edge e: graph[x])
    {
        if(e.v==last) continue;

        route2Edges[root].push_back(e.ind);

        bool res = DFS(e.v, x, root, destination);
        if(res==true)
        {
            return true;
        }

        route2Edges[root].pop_back();
    }

    return false;
}

bool toSkip[MAXN];
void Init()
{
    for(int i = 0;i<m;i++)
    {
        if(routes[i].first>routes[i].second) swap(routes[i].first, routes[i].second);
        for(int j = 0;j<i;j++)
        {
            if(routes[j]==routes[i])
            {
                toSkip[i] = true;
                break;
            }
        }
        if(toSkip[i]==true) continue;

        DFS(routes[i].first, -1, i, routes[i].second);

        //cout << " --- " << i << " ---- " << '\n';
        //for(int ind: route2Edges[i]) cout << ind << " ";
        //cout << '\n';
    }
}

bool used[MAXN];

bool check(int ind, int invert)
{
    for(int edgeInd: route2Edges[ind])
    {
        if(used[(edgeInd^invert)^1]==true) return false;
    }

    return true;
}
void apply(int ind, int invert, bool sign)
{
    for(int edgeInd: route2Edges[ind])
    {
        used[(edgeInd^invert)] = sign;
    }
}

int bestCMax = MAXN;
int bestC[MAXN];

ConfigurationBuilder cb;
void bruteForce(int ind)
{
    if(ind==m)
    {
        cb.reset();
        for(int i = 0;i<2*n;i++)
        {
            if(used[i]==true)
            {
                cb.addEdge(edges[i]);
            }
        }

        cb.calc();
        //cout << "OK " << cb.cMax << '\n';

        if(cb.cMax<bestCMax)
        {
            bestCMax = cb.cMax;
            for(int i = 1;i<=n;i++)
            {
                bestC[i] = cb.c[i];
            }
        }

        return;
    }
    if(toSkip[ind]==true)
    {
        bruteForce(ind+1);
        return;
    }

    for(int invert = 0;invert<2;invert++)
    {
        if(check(ind, invert)==true)
        {
            //cout << ind << " -- " << invert << '\n';

            apply(ind, invert, true);
            bruteForce(ind+1);
            apply(ind, invert, false);
        }
    }
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> m >> T;
    for(int i = 0;i<n-1;i++)
    {
        int u, v;
        cin >> u >> v;

        edges.push_back(Edge(u, v, edges.size()));
        graph[u].push_back(edges.back());

        //cout << edges.back().u << " -> " << edges.back().v << " || " << edges.back().ind << '\n';

        edges.push_back(Edge(v, u, edges.size()));
        graph[v].push_back(edges.back());

        //cout << edges.back().u << " -> " << edges.back().v << " || " << edges.back().ind << '\n';
    }
    for(int i = 0;i<m;i++)
    {
        cin >> routes[i].first >> routes[i].second;
    }

    Init();
    bruteForce(0);

    if(bestCMax==MAXN)
    {
        cout << "-1" << '\n';
        return 0;
    }

    cout << bestCMax << '\n';

    cout << bestC[1];
    for(int i = 2;i<=n;i++)
    {
        cout << " " << bestC[i];
    }
    cout << '\n';

    return 0;
}
/*
5 3 0
1 2
1 3
1 4
1 5

1 2
1 3
4 2
*/
