#include <iostream>
#include <algorithm>
#include <vector>
#include <set>
#include <tuple>

using namespace std;

vector <int> p;
void dfs1(const vector <set <int>>& g, vector <bool>& used, int v, int h, vector <pair <int, int>>& arr)
{
	used[v] = true;
	for (int u : g[v])
	{
		if (!used[u])
		{
			arr.push_back({ h, v });
			p[u] = v;
			dfs1(g, used, u, h + 1, arr);
		}
	}
	
	arr.push_back({ h, v });
}

void dfs2(const vector <set <int>>& g, vector <bool>& used, int v, vector <int> &d, vector <int> &prev)
{
	used[v] = true;
	for (int u : g[v])
	{
		if (!used[u])
		{
			d[u] = d[v] + 1;
			prev[u] = v;
			dfs2(g, used, u, d, prev);
		}
	}
}

void dfs3(const vector <set <int>>& g, vector <bool>& used, int v, vector <int>& c)
{
	used[v] = true;
	for (int u : g[v])
	{
		if (!used[u])
		{
			c[u] = c[v] - 1;
			dfs3(g, used, u, c);
		}
	}
}

vector <vector <pair <int, int>>> st;
vector <int> floor_log, first;

void build_lca(const vector <set <int>>& g)
{
	vector <bool> used(g.size(), false);
	vector <pair <int, int>> vis;
	dfs1(g, used, 0, 0, vis);

	st.resize(vis.size());
	floor_log.resize(vis.size() + 1);
	int log_cnt = 0;
	for (int i = 0; i <= vis.size(); ++i)
	{
		if (i >> (log_cnt + 1))
			++log_cnt;
		floor_log[i] = log_cnt;
	}

	for (int i = 0; i < st.size(); ++i)
		st[i].resize(log_cnt + 1);

	for (int i = 0; i < vis.size(); ++i)
		st[i][0] = vis[i];
	for (int i = 0; i < log_cnt; ++i)
		for (int j = 0; j + (1 << i) < st.size(); ++j)
			st[j][i + 1] = min(st[j][i], st[j + (1 << i)][i]);

	first.assign(g.size(), -1);
	for (int i = 0; i < vis.size(); ++i)
	{
		int v = vis[i].second;
		if (first[v] == -1)
			first[v] = i;
	}
}

int get_lca(int u, int v)
{
	int l = first[u], r = first[v];
	if (l > r)
		swap(l, r);
	++r;
	int log = floor_log[r - l];
	pair <int, int> ans = min(st[l][log], st[r - (1 << log)][log]);
	return ans.second;
}

bool add_branch(vector <set <int>> &g, int s, int t)
{
	int x = get_lca(s, t);
	
	while (s != x)
	{
		if (g[s].count(p[s]) == 0)
			return false;
		g[p[s]].erase(s);
		s = p[s];
	}

	while (t != x)
	{
		if (g[p[t]].count(t) == 0)
			return false;
		g[t].erase(p[t]);
		t = p[t];
	}

	return true;
}

bool check(const vector <bool>& used)
{
	for (int i = 0; i < used.size(); ++i)
		if (!used[i])
			return true;
	return false;
}

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0), cout.tie(0);

	int n = 0, m = 0, t = 0;
	cin >> n >> m >> t;

	vector <set <int>> g(n);
	p.assign(n, 0);
	for (int i = 1; i < n; ++i)
	{
		int u = 0, v = 0;
		cin >> u >> v;
		--u, --v;
		
		g[u].insert(v);
		g[v].insert(u);
	}

	build_lca(g);

	vector <pair <int, int>> br(m);
	for (int i = 0; i < m; ++i)
		cin >> br[i].first >> br[i].second;

	for (int i = 0; i < m; ++i)
	{
		if (!add_branch(g, --br[i].first, --br[i].second))
		{
			cout << -1;
			return 0;
		}
	}

	vector <vector <int>> dist(n, vector <int>(n, -1)), prev(n, vector <int>(n, -1));
	for (int i = 0; i < n; ++i)
	{
		vector <bool> used(n, false);
		dist[i][i] = 0;
		dfs2(g, used, i, dist[i], prev[i]);
	}

	int x = 0, y = 0;
	for (int i = 0; i < n; ++i)
		for (int j = 0; j < n; ++j)
			if (dist[i][j] > dist[x][y])
				tie(x, y) = tie(i, j);

	cout << dist[x][y] + 1 << '\n';

	if (t == 1)
	{
		vector <set <int>> rev_g(n);
		for (int u = 0; u < n; ++u)
			for (int v : g[u])
				rev_g[v].insert(u);

		vector <int> c(n, -1);
		vector <bool> used(n, false);
		while (check(used))
		{
			x = y = 0;
			for (int i = 0; i < n; ++i)
			{
				if (used[i])
					continue;

				for (int j = 0; j < n; ++j)
				{
					if (used[j])
						continue;
					if (dist[i][j] > dist[x][y])
						tie(x, y) = tie(i, j);
				}
			}

			c[y] = dist[x][y] + 1;
			dfs3(rev_g, used, y, c);
			dfs3(g, used, y, c);
		}

		for (int i = 0; i < n; ++i)
			cout << c[i] << ' ';
	}

	return 0;
}