#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;
const int N = 500000;
const int logN = 20;

int up[N][logN], p[N], tin[N], tout[N], h[N], tm = 0;
vector<int> g[N], gg[N];

void dfs(int v, int hh = 0, int pp = -1)
{
    p[v] = pp, tin[v] = tm++, h[v] = hh;
    for (int u: g[v])
        if (u - pp)
            dfs(u, hh + 1, v);
    tout[v] = tm++;
}

void precalc_up(int n)
{
    for (int i = 0; i < n; ++i)
        up[i][0] = p[i];
    for (int j = 1; j < logN; ++j)
        for (int i = 0; i < n; ++i)
            up[i][j] = up[i][j-1] == -1 ? -1 : up[up[i][j-1]][j-1];
}

inline bool is_anc(int a, int b) {return a == -1 ? 1 : b == -1 ? 0 : tin[a] <= tin[b] && tout[b] <= tout[a];}

int lca(int a, int b)
{
    if (is_anc(a, b))
        return a;
    if (is_anc(b, a))
        return b;
    for (int i = logN - 1; i >= 0; --i)
        if (!is_anc(up[a][i], b))
            a = up[a][i];
    return p[a];
}

inline int dst(int a, int b) {return h[a] + h[b] - 2 * h[lca(a,b)];}

vector<int> comp[N];
int comp_id[N], sz[N], c_comp[N]; // size m
static bool used_comp[N];

void init_dsu(int n)
{
    for (int i = 0; i < n; ++i)
        comp[i] = {comp_id[i] = i}, sz[i] = 1;
}

void merge(int a, int b)
{
    a = comp_id[a], b = comp_id[b];
    if (a == b)
        return;
    if (sz[a] < sz[b])
        swap(a, b);
    for (int x: comp[b])
        comp[a].push_back(x), comp_id[x] = a;
    sz[a] += sz[b], sz[b] = 0;
}

const int S = 1000;
int eq_dir[S][S]; // 1 - coincides, -1 - different

void bi_dfs(int v, int m, int color = 0)
{
    c_comp[v] = color, used_comp[v] = true;
    for (int u = 0; u < m; ++u)
        if (!used_comp[u] && sz[u] && eq_dir[comp[v][0]][comp[u][0]] == -1)
            bi_dfs(u, m, !color);
}

int color[N];

bool check_bipartite(int m)
{
    for (int i = 0; i < m; ++i)
        if (sz[i] && !used_comp[i])
            bi_dfs(i, m);
    for (int i = 0; i < m; ++i)
        for (int j = 0; j < m; ++j)
            if (sz[i] && sz[j] && eq_dir[comp[i][0]][comp[j][0]] == -1 && c_comp[i] == c_comp[j])
                return false;
    for (int i = 0; i < m; ++i)
        if (sz[i])
            for (int j: comp[i])
                color[j] = c_comp[i];
    return true;
}

static bool used[N];
vector<int> ts;

void ts_dfs(int v)
{
    used[v] = true;
    for (int u: gg[v])
        if (!used[u])
            ts_dfs(u);
    ts.push_back(v);
}

int main()
{
    int n, m, call_t, a, b;
    cin >> n >> m >> call_t;
    for (int i = 1; i < n; ++i)
        cin >> a >> b, g[--a].push_back(--b), g[b].push_back(a);
    dfs(0);
    precalc_up(n);
    int path_a[m], path_b[m], path_len_m1[m];
    for (int i = 0; i < m; ++i)
        cin >> path_a[i] >> path_b[i], path_len_m1[i] = dst(path_a[i], path_b[i]) - 1;
    for (int i = 0; i < m; ++i)
        if (path_a[i] > path_b[i])
            swap(path_a[i], path_b[i]);
    for (int i = 0; i < m; ++i)
    {
        for (int j = 0; j < m; ++j)
        {
            eq_dir[i][j] = i == j;
            if (i == j)
                continue;
            for (int k = 1; k < n; ++k)
            {
                int or1 = 0, or2 = 0;
                if (dst(path_a[i], k) + dst(path_b[i], p[k]) == path_len_m1[i])
                    or1 = 1;
                if (dst(path_a[i], p[k]) + dst(path_b[i], k) == path_len_m1[i])
                    or1 = -1;
                if (dst(path_a[j], k) + dst(path_b[j], p[k]) == path_len_m1[j])
                    or2 = 1;
                if (dst(path_a[j], p[k]) + dst(path_b[j], k) == path_len_m1[j])
                    or2 = -1;
                if (!or1 || !or2)
                    continue;
                int supp_eq_dir = or1 == 1 ^ or2 == 1 ? -1 : 1;
                if (eq_dir[i][j] && eq_dir[i][j] != supp_eq_dir)
                {
                    cout << -1;
                    return 0;
                }
                eq_dir[i][j] = supp_eq_dir;
            }
        }
    }
    init_dsu(m);
    for (int i = 0; i < m; ++i)
        for (int j = 0; j < m; ++j)
            if (eq_dir[i][j] == 1)
                merge(i, j);
    if (!check_bipartite(m))
    {
        cout << -1;
        return 0;
    }
    int ans[n];
    fill(ans, ans + n, -1);
    for (int i = 1; i < n; ++i)
    {
        int orr = 1;
        for (int j = 0; j < m; ++j)
        {
            if (dst(path_a[j], i) + dst(path_b[j], p[j]) == path_len_m1[j] && color[j]
                    || dst(path_a[j], p[i]) + dst(path_b[j], i) == path_len_m1[j] && !color[j])
                orr = 1;
            else if (dst(path_a[j], p[i]) + dst(path_b[j], i) == path_len_m1[j] && color[j]
                    || dst(path_a[j], i) + dst(path_b[j], p[i]) == path_len_m1[j] && !color[j])
                orr = -1;
        }
        if (orr == 1)
            gg[i].push_back(p[i]);
        if (orr == -1)
            gg[p[i]].push_back(i);
    }
    for (int i = 0; i < n; ++i)
        if (!used[i])
            ts_dfs(i);
    for (int x: ts)
    {
        ans[x] = 0;
        for (int y: gg[x])
            ans[x] = max(ans[x], ans[y] + 1);
    }
    int y = *max_element(ans, ans + n);
    cout << y + 1 << '\n';
    if (call_t)
    {
        for (int i = 0; i < n; ++i)
            cout << y + 1 - ans[i] << ' ';
    }
    return 0;
}
