#define PYHAEM_S_NOGI 0

#if PYHAEM_S_NOGI >= 1
#pragma GCC optimize("Ofast")
#pragma GCC target("tune=native,sse,sse2,sse3,avx,mmx")
#pragma GCC optimize("unroll-loops,no-stack-protector,btr-bb-exclusive,fast-math")
#endif

#include <bits/stdc++.h>

using namespace std;

#define int long long

#if PYHAEM_S_NOGI >= 2
const int BUFFER_SIZE = 65536;
char read_buffer[BUFFER_SIZE];
char* read_buffer_ptr = read_buffer;
char* read_buffer_end = read_buffer;
int getc_buffered() {
	if(read_buffer_ptr == read_buffer_end) {
		read_buffer_ptr = read_buffer;
		read_buffer_end = read_buffer + fread(read_buffer, 1, BUFFER_SIZE, stdin);
	 	if(read_buffer_ptr == read_buffer_end) {
			return -1;
		}
	}
	return *read_buffer_ptr++;
}
int readint() {
	int n = 0;
	int sign = 1;
	char c;
	do {
		c = getc_buffered();
	} while(!(('0' <= c && c <= '9') || c == '-'));
	if(c == '-') {
		sign = -1;
		c = getc_buffered();
	}
	do {
		n = n * 10 + (c - '0');
		c = getc_buffered();
	} while('0' <= c && c <= '9');
	return n;
}
#else
int readint() {
	int n;
	cin >> n;
	return n;
}
#endif

#if PYHAEM_S_NOGI >= 3
const int MAX_MEMORY = 300 * 1024 * 1024;
char MEMORY[MAX_MEMORY];
char* memory_ptr = MEMORY;

void* operator new(size_t sz) {
	void* res = memory_ptr;
	memory_ptr += sz;
	assert(memory_ptr < MEMORY + MAX_MEMORY);
	return res;
}

void operator delete(void*) {}
#endif

#if PYHAEM_S_NOGI >= 4
class DynamicArray {
public:
	int sz;
	int cap;
	int* ptr;

	DynamicArray() {
		sz = 0;
		cap = 16;
		ptr = new int[cap];
	}

	void push_back(int x) {
		if(sz == cap) {
			cap *= 2;
			int* new_ptr = new int[cap];
			memmove(new_ptr, ptr, sz * sizeof(int));
			delete[] ptr;
			ptr = new_ptr;
		}
		ptr[sz++] = x;
	}

	int size() {
		return sz;
	}

	int& operator[] (int i) {
		return ptr[i];
	}
};
#else
typedef vector<int> DynamicArray;
#endif

class SegmentTree {
public:
	vector<int> a;
	int n;

	SegmentTree(int sz) {
		n = sz;
		a.resize(4 * n);
	}

	int get(int v, int vl, int vr, int l, int r) {
		if(vl == l && vr == r) {
			return a[v];
		}
		int vm = (vl + vr) / 2;
		if(r <= vm) {
			return get(v * 2, vl, vm, l, r);
		} else if(l >= vm) {
			return get(v * 2 + 1, vm, vr, l, r);
		} else {
			return get(v * 2, vl, vm, l, vm) + get(v * 2 + 1, vm, vr, vm, r);
		}
	}
	int get(int l, int r) {
		return get(1, 0, n, l, r);
	}
	int get(int i) {
		return get(i, i + 1);
	}

	void set(int v, int vl, int vr, int i, int x) {
		if(vr - vl == 1) {
			a[v] = x;
			return;
		}
		int vm = (vl + vr) / 2;
		if(i < vm) {
			set(v * 2, vl, vm, i, x);
		} else {
			set(v * 2 + 1, vm, vr, i, x);
		}
	}
	void set(int i, int x) {
		set(1, 0, n, i, x);
	}
};

class Vertex {
public:
	int y, sz, value;
	Vertex* left;
	Vertex* right;
	Vertex(int x) {
		y = rand() ^ (rand() << 16);
		sz = 1;
		value = x;
		left = nullptr;
		right = nullptr;
	}
	void pull() {
		sz = (left ? left->sz : 0) + 1 + (right ? right->sz : 0);
	}
};

class Treap {
public:
	Vertex* root;
	Treap() {
		root = nullptr;
	}

	Vertex* merge(Vertex* a, Vertex* b) {
		if(a == nullptr) {
			return b;
		}
		if(b == nullptr) {
			return a;
		}
		if(a->y > b->y) {
			a->right = merge(a->right, b);
			a->pull();
			return a;
		} else {
			b->left = merge(a, b->left);
			b->pull();
			return b;
		}
	}

	pair<Vertex*, Vertex*> split(Vertex* v, int pos) {
		if(v == nullptr) {
			return make_pair(nullptr, nullptr);
		}
		if(pos >= (v->left ? v->left->sz : 0) + 1) {
			auto p = split(v->right, pos - (v->left ? v->left->sz : 0) - 1);
			v->right = p.first;
			v->pull();
			return make_pair(v, p.second);
		} else {
			auto p = split(v->left, pos);
			v->left = p.second;
			v->pull();
			return make_pair(p.first, v);
		}
	}
};

vector<int> z_function(string s) {
	int n = s.size();
	vector<int> z(n);
	int l = 0;
	int r = 0;
	for(int i = 1; i < n; i++) {
		if(i < r) {
			z[i] = min(r - i, z[i - l]);
		}
		while(i + z[i] < n && s[i + z[i]] == s[z[i]]) {
			z[i]++;
		}
		if(i + z[i] > r) {
			l = i;
			r = i + z[i];
		}
	}
	return z;
}

vector<int> pref_function(string s) {
	int n = s.size();
	vector<int> pref(n);
	for(int i = 1; i < n; i++) {
		pref[i] = pref[i - 1];
		while(pref[i] >= 1 && s[pref[i]] != s[i]) {
			pref[i] = pref[pref[i] - 1];
		}
		if(s[pref[i]] == s[i]) {
			pref[i]++;
		}
	}
	return pref;
}





vector< vector<int> > adj, adj1;
vector< vector<int> > paths;
vector<SegmentTree> path_sts;
vector<int> sz;
vector< vector< pair<int, int> > > actions;
vector<int> tin;
vector<int> tout;
vector< vector<int> > up;
vector<bool> used1;
vector<int> color1;
vector<int> values;
int curtime = 0;
void dfs1(int u, int p) {
	tin[u] = curtime++;
	up[u][0] = p;
	for(int i = 1; i < 20; i++) {
		up[u][i] = up[u][i - 1] == -1 ? -1 : up[up[u][i - 1]][i - 1];
	}
	sz[u] = 1;
	for(int v: adj[u]) {
		if(v != p) {
			dfs1(v, u);
			sz[u] += sz[v];
		}
	}
	tout[u] = curtime++;
}
void dfs2(int u, int p, int path) {
	int nextv = -1;
	for(int v: adj[u]) {
		if(v != p && (nextv == -1 || sz[v] > sz[nextv])) {
			nextv = v;
		}
	}
	if(nextv != -1) {
		paths[path].push_back(nextv);
		dfs2(nextv, u, path);
	}
	for(int v: adj[u]) {
		if(v != p && v != nextv) {
			int newpath = paths.size();
			paths.emplace_back();
			paths.back().push_back(v);
			dfs2(v, u, newpath);
		}
	}
}
bool isin(int u, int v) {
	return tin[u] >= tin[v] && tout[u] <= tout[v];
}
int lca(int u, int v) {
	if(isin(u, v)) {
		return v;
	}
	if(isin(v, u)) {
		return u;
	}
	for(int i = 19; i >= 0; i--) {
		if(up[v][i] != -1 && !isin(u, up[v][i])) {
			v = up[v][i];
		}
	}
	return up[v][0];
}
int preparent(int u, int v) {
	assert(isin(u, v));
	if(v == u) {
		return -1;
	}
	for(int i = 19; i >= 0; i--) {
		if(up[u][i] != -1 && isin(up[u][i], v) && up[u][i] != v) {
			u = up[u][i];
		}
	}
	return u;
}
bool dfs4(int u, int c) {
	used1[u] = true;
	color1[u] = c;
	for(int v: adj1[u]) {
		if(color1[v] != -1 && color1[v] == c) {
			return false;
		}
		if(!used1[v] && !dfs4(v, !c)) {
			return false;
		}
	}
	return true;
}
void hldset(int v, int u, int c) {
	while(v != u) {
		color1[v] = c;
		v = up[v][0];
	}
}
void dfs3(int u, int p) {
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		adj1[v].clear();
		used1[v] = false;
	}
	for(auto [v1, v2]: actions[u]) {
		v1 = preparent(v1, u);
		v2 = preparent(v2, u);
		if(v1 != -1 && v2 != -1) {
			adj1[v1].push_back(v2);
			adj1[v2].push_back(v1);
		}
	}
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		if(color1[v] != -1) {
			if(!dfs4(v, color1[v])) {
				cout << -1 << endl;
				exit(0);
			}
		}
	}
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		if(!used1[v]) {
			if(!dfs4(v, 0)) {
				cout << -1 << endl;
				exit(0);
			}
		}
	}
	for(auto [v1, v2]: actions[u]) {
		if(preparent(v1, u) != -1) {
			int c1 = color1[preparent(v1, u)];
			hldset(v1, u, c1);
		}
		if(preparent(v2, u) != -1) {
			int c2 = color1[preparent(v2, u)];
			hldset(v2, u, c2);
		}
	}
	for(int v: adj[u]) {
		if(v != p) {
			dfs3(v, u);
		}
	}
}
void dfs5(int u, int p) {
	if(p != -1) {
		values[u] = values[p] + (color1[u] == 1 ? 1 : -1);
	}
	for(int v: adj[u]) {
		if(v != p) {
			dfs5(v, u);
		}
	}
}






void slowdfs(int u, int p, int mask) {
	if(p != -1) {
		values[u] = values[p] + (((mask >> (u - 1)) & 1) ? 1 : -1);
	}
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		slowdfs(v, u, mask);
	}
}
bool slowdfs1(int u, int p, int w) {
	if(u == w) {
		return true;
	}
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		if(slowdfs1(v, u, w)) {
			return values[u] > values[v];
		}
	}
	return false;
}
bool slowdfs2(int u, int p, int w) {
	if(u == w) {
		return true;
	}
	for(int v: adj[u]) {
		if(v == p) {
			continue;
		}
		if(slowdfs2(v, u, w)) {
			return values[u] < values[v];
		}
	}
	return false;
}



signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	int n, m, t;
	cin >> n >> m >> t;
	adj.resize(n);
	for(int i = 0; i < n - 1; i++) {
		int u, v;
		cin >> u >> v;
		u--; v--;
		adj[u].push_back(v);
		adj[v].push_back(u);
	}





	if(m <= 15 && n <= 15) {
		vector< pair<int, int> > actions;
		for(int i = 0; i < m; i++) {
			int u, v;
			cin >> u >> v;
			u--; v--;
			actions.emplace_back(u, v);
		}
		values.resize(n);
		int k = 1000000000;
		int bestmask;
		for(int mask = 0; mask < (1 << (n - 1)); mask++) {
			values[0] = 0;
			slowdfs(0, -1, mask);
			bool ok = true;
			for(auto [u, v]: actions) {
				if(!slowdfs1(u, -1, v) && !slowdfs2(u, -1, v)) {
					ok = false;
					break;
				}
			}
			if(!ok) {
				continue;
			}
			int minval = 0, maxval = 0;
			for(int i = 0; i < n; i++) {
				minval = min(minval, values[i]);
				maxval = max(maxval, values[i]);
			}
			int curk = maxval - minval + 1;
			if(curk < k) {
				k = curk;
				bestmask = mask;
			}
		}
		if(k == 1000000000) {
			cout << -1 << endl;
			return 0;
		}
		cout << k << endl;
		values[0] = 0;
		slowdfs(0, -1, bestmask);
		int minval = 0, maxval = 0;
		for(int i = 0; i < n; i++) {
			minval = min(minval, values[i]);
			maxval = max(maxval, values[i]);
		}
		for(int i = 0; i < n; i++) {
			cout << values[i] - minval + 1 << " ";
		}
		cout << endl;
		return 0;
	}





	sz.resize(n);
	tin.resize(n);
	tout.resize(n);
	up.resize(n, vector<int>(20));
	dfs1(0, -1);
	paths.emplace_back();
	dfs2(0, -1, 0);

	for(int i = 0; i < paths.size(); i++) {
		path_sts.emplace_back(paths[i].size());
	}

	actions.resize(n);
	for(int i = 0; i < m; i++) {
		int u, v;
		cin >> u >> v;
		u--; v--;
		int w = lca(u, v);
		actions[w].emplace_back(u, v);
	}

	adj1.resize(n);
	used1.resize(n);
	color1.resize(n, -1);
	dfs3(0, -1);

	values.resize(n);
	dfs5(0, -1);
	int minval = 0;
	int maxval = 0;
	for(int i = 0; i < n; i++) {
		minval = min(minval, values[i]);
		maxval = max(maxval, values[i]);
	}
	cout << maxval - minval + 1 << endl;
	for(int i = 0; i < n; i++) {
		cout << values[i] - minval + 1 << " ";
	}
	cout << endl;

#if PYHAEM_S_NOGI >= 3
	cerr << endl << endl << "Memory used: " << (double)((memory_ptr - MEMORY) * 10000 / MAX_MEMORY) / 100 << "%" << endl;
#endif

	return 0;
}