#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

typedef long long ll;

struct edge
{
    int from, to, idx;
};

int n, m;
vector<vector<edge>> g;
vector<vector<pair<int, int>>> paths_on_edge;
vector<pair<int, int>> paths;
vector<edge> cur;
vector<vector<pair<int, int>>> gc;
vector<int> color;
bool flag = false;
vector<vector<bool>> more_than;
vector<int> price;
vector<int> tin;
vector<int> tout;
vector<int> parent;
int timer = 0;
vector<vector<int>> gd;

void dfs(int u, int prev, int pnum)
{
    if (!cur.empty())
    {
        int start = cur[0].from;
        int end = u;
        if (start == paths[pnum].first && end == paths[pnum].second)
        {
            for (auto p : cur)
            {
                if (p.from < p.to)
                    paths_on_edge[p.idx].push_back({pnum, 0});
                else
                    paths_on_edge[p.idx].push_back({pnum, 1});
            }
        }
        else if (end == paths[pnum].first && start == paths[pnum].second)
        {
            for (auto p : cur)
            {
                if (p.from < p.to)
                    paths_on_edge[p.idx].push_back({pnum, 1});
                else
                    paths_on_edge[p.idx].push_back({pnum, 0});
            }
        }
    }

    for (auto e : g[u])
    {
        if (e.to != prev)
        {
            cur.push_back(e);
            dfs(e.to, u, pnum);
            cur.pop_back();
        }
    }
}

void dfs_color(int u)
{
    for (auto v : gc[u])
    {
        if (color[v.first] == -1)
        {
            if (v.second == 0)
                color[v.first] = color[u];
            else
                color[v.first] = 1 - color[u];

            dfs_color(v.first);
        }
        else
        {
            if ((v.second == 0 && color[v.first] != color[u]) ||
                (v.second == 1 && color[v.first] == color[u]))
                flag = true;
        }
    }
}

void dfs_price(int u, int prev)
{
    for (auto e : g[u])
    {
        if (e.to == prev)
            continue;

        if (!paths_on_edge[e.idx].empty())
        {
            if ((paths_on_edge[e.idx][0].second +
                 color[paths_on_edge[e.idx][0].first] +
                 (e.from > e.to)) % 2 == 0)
                more_than[e.to][u] = true;
            else
                more_than[u][e.to] = true;
        }

        dfs_price(e.to, u);
    }
}

int calc(int u)
{
    if (price[u] == -1)
    {
        price[u] = 1;
        for (int i = 0; i < n; i++)
            if (more_than[u][i])
                price[u] = max(price[u], calc(i) + 1);
    }

    return price[u];
}

void dfs_timer(int u, int prev)
{
    if (prev == -1)
        parent[u] = u;
    else
        parent[u] = prev;
    tin[u] = timer++;

    for (edge e : g[u])
        if (e.to != prev)
            dfs_timer(e.to, u);

    tout[u] = timer++;
}

bool is_parent(int u, int v)
{
    return tin[u] <= tin[v] && tout[v] <= tout[u];
}

int calc_fast(int u)
{
    if (price[u] == -1)
    {
        price[u] = 1;
        for (int v : gd[u])
            price[u] = max(price[u], calc_fast(v) + 1);
    }

    return price[u];
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int t;
    cin >> n >> m >> t;

    g.resize(n);
    for (int i = 0; i < n - 1; i++)
    {
        int a, b;
        cin >> a >> b;
        a--;
        b--;

        g[a].push_back({a, b, i});
        g[b].push_back({b, a, i});
    }

    paths.resize(m);
    for (int i = 0; i < m; i++)
    {
        cin >> paths[i].first >> paths[i].second;
        paths[i].first--;
        paths[i].second--;
    }

    if (n <= 15 && m <= 15)
    {
        paths_on_edge.resize(n - 1);
        for (int i = 0; i < m; i++)
            dfs(paths[i].first, -1, i);

        gc.resize(m);
        for (int i = 0; i < n - 1; i++)
        {
            for (int j = 0; j < paths_on_edge[i].size(); j++)
            {
                for (int k = j + 1; k < paths_on_edge[i].size(); k++)
                {
                    if (paths_on_edge[i][j].second == paths_on_edge[i][k].second)
                    {
                        gc[paths_on_edge[i][j].first].push_back({
                                                                        paths_on_edge[i][k].first, 0});
                        gc[paths_on_edge[i][k].first].push_back({
                                                                        paths_on_edge[i][j].first, 0});
                    } else
                    {
                        gc[paths_on_edge[i][j].first].push_back({
                                                                        paths_on_edge[i][k].first, 1});
                        gc[paths_on_edge[i][k].first].push_back({
                                                                        paths_on_edge[i][j].first, 1});
                    }
                }
            }
        }

        color.resize(m, -1);
        pair<int, vector<int>> best = {1e9, {}};
        for (int mask = 0; mask < (1 << m); mask++)
        {
            for (int i = 0; i < m; i++)
                color[i] = ((mask & (1 << i)) >> i);

            flag = false;
            for (int i = 0; i < m; i++)
            {
                dfs_color(i);
                if (flag)
                    break;
            }

            if (flag)
                continue;

            more_than.clear();
            more_than.resize(n, vector<bool>(n, false));
            dfs_price(0, -1);

            vector<bool> start_from(n, true);
            for (int i = 0; i < n; i++)
                for (int j = 0; j < n; j++)
                    if (more_than[i][j])
                        start_from[j] = false;

            price.clear();
            price.resize(n, -1);
            for (int i = 0; i < n; i++)
                if (start_from[i])
                    calc(i);

            best = min(best, {*max_element(price.begin(), price.end()), price});
        }

        if (best.first == 1e9)
            cout << "-1" << endl;
        else
        {
            cout << best.first << endl;
            for (int x : best.second)
                cout << x << " ";
        }
    }
    else
    {
        tin.resize(n);
        tout.resize(n);
        parent.resize(n);
        dfs_timer(0, -1);

        gd.resize(n);
        vector<bool> into(n, false);
        for (int i = 0; i < m; i++)
        {
            int u = paths[i].first;
            int v = paths[i].second;

            if (is_parent(u, v))
            {
                while (v != u)
                {
                    gd[parent[v]].push_back(v);
                    into[v] = true;
                    v = parent[v];
                }
            }
            else if (is_parent(v, u))
            {
                while (v != u)
                {
                    gd[parent[u]].push_back(u);
                    into[u] = true;
                    u = parent[u];
                }
            }
            else
            {
                while (!is_parent(u, v))
                {
                    gd[parent[u]].push_back(u);
                    into[u] = true;
                    u = parent[u];
                }

                int c = v;
                while (c != u)
                {
                    gd[parent[c]].push_back(c);
                    into[c] = true;
                    c = parent[c];
                }
            }
        }

        price.resize(n, -1);
        for (int i = 0; i < n; i++)
            if (!into[i])
                calc_fast(i);

        cout << *max_element(price.begin(), price.end()) << endl;
        for (int x : price)
            cout << x << " ";
    }

    return 0;
}