#include <iostream>
#include <vector>
#include <map>
#include <algorithm>
#include <cassert>

using namespace std;

const int LOG = 20;

vector <int> dep;
vector <int> parent[LOG];

void dfs_build(int v, vector <vector <int>>& G, int p = 0, int h = 0) {
	dep[v] = h;
	parent[0][v] = p;
	for (auto u : G[v]) {
		if (u != p)
			dfs_build(u, G, v, h + 1);
	}
}

void build(int n, vector <vector <int>>& G) {
	dep.resize(n);
	for (int i = 0; i < LOG; i++)
		parent[i].resize(n);
	dfs_build(0, G);
	for (int i = 1; i < LOG; i++) {
		for (int j = 0; j < n; j++) {
			parent[i][j] = parent[i - 1][parent[i - 1][j]];
		}
	}
}

int lca(int u, int v) {
	if (dep[u] < dep[v])
		swap(u, v);
	for (int i = LOG - 1; i > -1; i--) {
		if (dep[u] - (1 << i) >= dep[v])
			u = parent[i][u];
	}
	for (int i = LOG - 1; i > -1; i--) {
		if (parent[i][u] != parent[i][v]) {
			u = parent[i][u], v = parent[i][v];
		}
	}
	return (u == v ? u : parent[0][u]);
}

int get_dist(int u, int v) {
	return dep[u] + dep[v] - 2 * dep[lca(u, v)];
}

void dfs(int v, vector <vector <pair <int, int>>>& G, vector <int>& ans, 
	int val, vector <int> &used, int k = -1) {
	ans[v] = val;
	used[v] = 1;
	for (auto u : G[v]) {
		if (!used[u.second]) {
			dfs(u.second, G, ans, val + k * u.first, used, -k);
		}
	}
}

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	int n, m, t;
	cin >> n >> m >> t;
	vector <vector <int>> G_pr(n);
	for (int i = 0; i < n - 1; i++) {
		int u, v;
		cin >> u >> v, u--, v--;
		G_pr[u].emplace_back(v);
		G_pr[v].emplace_back(u);
	}
	build(n, G_pr);
	vector <vector <pair <int, int>>> G(n);
	vector <int> term(n, 0);
	for (int i = 0; i < m; i++) {
		int u, v;
		cin >> u >> v, u--, v--;
		term[u]++, term[v]++;
		int dist = get_dist(u, v);
		G[u].emplace_back(dist, v);
		G[v].emplace_back(dist, u);
	}
	int ans = 0;
	vector <pair <int, int>> call;
	for (int i = 0; i < n; i++) {
		if (!G[i].size()) continue;
		call.emplace_back(-max_element(G[i].begin(), G[i].end())->first, i);
		if (ans < -call.back().first) {
			ans = -call.back().first;
		}
	}
	sort(call.begin(), call.end());
	cout << ++ans << '\n';
	vector <int> res(n, 1);
	vector <int> used(n, 0);
	for (int i = 0; i < call.size(); i++) {
		if (!used[call[i].second])
			dfs(call[i].second, G, res, -call[i].first + 1, used);
	}
	for (int i = 0; i < n; i++) {
		for (auto u : G[i]) {
			if (res[i] > res[u.second]) {
				int tmp = i, cur_res = res[i];
				int gl = lca(i, u.second);
				while (dep[tmp] > dep[gl]) {
					res[tmp] = cur_res--;
					tmp = parent[0][tmp];
				}
				tmp = u.second, cur_res = res[u.second];
				while (dep[tmp] > dep[gl]) {
					res[tmp] = cur_res++;
					tmp = parent[0][tmp];
				}
				res[gl] = cur_res;
			}
		}
	}
	for (auto u : res) {
		cout << u << ' ';
	}
}

/*
7 5 1
1 2
2 3
3 4
3 5
2 6
6 7
1 2
3 6
6 7
3 4
5 3*/