#include<iostream>
#include<vector>
#include<time.h>
#include<set>
#include<algorithm>

using namespace std;

double startTime;
const int MAXN = 2e5 + 5;

struct Edge
{
    int u, v;
    int ind;

    Edge(){}
    Edge(int u, int v, int ind)
    {
        this->u = u;
        this->v = v;
        this->ind = ind;
    }
};

int n, m, T;
vector <Edge> edges;
pair <int, int> routes[MAXN];

vector <Edge> graph[MAXN];
vector <int> route2Edges[MAXN], route2Nodes[MAXN];

struct ConfigurationBuilder
{
    bool used[MAXN];
    vector <int> graph[MAXN];

    int cMax;
    int c[MAXN];

    void reset()
    {
        cMax = 0;
        topSort.clear();
        for(int i = 1;i<=n;i++)
        {
            c[i] = 0;
            used[i] = false;
            graph[i].clear();
        }
    }

    void addEdge(Edge e)
    {
        graph[e.u].push_back(e.v);
    }

    vector <int> topSort;
    void DFS(int x)
    {
        used[x] = true;
        for(int y: graph[x])
        {
            if(used[y]==true) continue;
            DFS(y);
        }

        topSort.push_back(x);
    }

    void calc()
    {
        for(int i = 1;i<=n;i++)
        {
            if(used[i]==false)
            {
                DFS(i);
            }
        }

        for(int x: topSort)
        {
            c[x] = 1;
            for(int y: graph[x])
            {
                c[x] = max(c[x], c[y]+1);
            }

            cMax = max(c[x], cMax);
        }
    }
};


bool DFS(int x, int last, int root, int destination)
{
    if(x==destination)
    {
        return true;
    }

    for(Edge e: graph[x])
    {
        if(e.v==last) continue;

        route2Edges[root].push_back(e.ind);

        bool res = DFS(e.v, x, root, destination);
        if(res==true)
        {
            return true;
        }

        route2Edges[root].pop_back();
    }

    return false;
}

int depth[MAXN];
int parent[MAXN], parentInd[MAXN];
int sparseParent[20][MAXN];

void DFSInit(int x, int last, int lastInd, int level)
{
    depth[x] = level;
    parent[x] = last;
    parentInd[x] = lastInd;

    for(Edge e: graph[x])
    {
        if(e.v==last) continue;
        DFSInit(e.v, x, e.ind, level+1);
    }
}

int findLCA(int u, int v)
{
    if(depth[u]>depth[v]) swap(u, v);

    for(int i = 19;i>=0;i--)
    {
        if(sparseParent[i][v]==-1) continue;
        if(depth[ sparseParent[i][v] ]>=depth[u])
        {
            v = sparseParent[i][v];
        }
    }

    if(u==v) return u;
    //cout << u << " %% " << v << '\n';

    for(int i = 19;i>=0;i--)
    {
        if(sparseParent[i][u]!=sparseParent[i][v])
        {
            u = sparseParent[i][u];
            v = sparseParent[i][v];
        }
    }

    return parent[v];
}

bool toSkip[MAXN];
void Init()
{
    DFSInit(1, -1, -1, 1);
    for(int x = 1;x<=n;x++)
    {
        //cout << x << " -> " << depth[x] << '\n';
        sparseParent[0][x] = parent[x];
    }
    for(int i = 1;i<20;i++)
    {
        for(int x = 1;x<=n;x++)
        {
            if(sparseParent[i-1][x]==-1)
            {
                sparseParent[i][x] = -1;
            }
            else
            {
                sparseParent[i][x] = sparseParent[i-1][ sparseParent[i-1][x] ];
            }
        }
    }

    set <pair <int, int>> seen;
    for(int i = 0;i<m;i++)
    {
        //cout << " --- " << i << " --- " << '\n';

        if(routes[i].first>routes[i].second) swap(routes[i].first, routes[i].second);
        if(seen.count(routes[i])==true)
        {
            toSkip[i] = true;
            continue;
        }

        seen.insert(routes[i]);

        int u = routes[i].first;
        int v = routes[i].second;
        int lca  = findLCA(u, v);
        //cout << "lca: " << lca << '\n';

        while(u!=lca)
        {
            route2Edges[i].push_back(parentInd[u]);
            route2Nodes[i].push_back(u);

            u = parent[u];
        }
        route2Nodes[i].push_back(lca);
        while(v!=lca)
        {
            route2Edges[i].push_back(parentInd[v]^1);
            route2Nodes[i].push_back(v);

            v = parent[v];
        }

        //for(int ind: route2Edges[i]) cout << ind << " ";
        //cout << '\n';
    }
}

int used[MAXN];
bool check(int ind, int invert)
{
    for(int edgeInd: route2Edges[ind])
    {
        if(used[(edgeInd^invert)^1]!=0) return false;
    }

    return true;
}
void apply(int ind, int invert, int sign)
{
    for(int edgeInd: route2Edges[ind])
    {
        used[(edgeInd^invert)] += sign;
    }
}

int bestCMax = MAXN;
int bestC[MAXN];

bool found = false;
ConfigurationBuilder cb;

void bruteForce(int ind)
{
    if(double(clock()-startTime)/CLOCKS_PER_SEC>5.6)
    {
        return;
    }
    if(T==0 && found==true)
    {
        return;
    }
    if(ind==m)
    {
        cb.reset();
        for(int i = 0;i<2*n;i++)
        {
            if(used[i]!=0)
            {
                cb.addEdge(edges[i]);
            }
        }

        cb.calc();
        if(cb.cMax<bestCMax)
        {
            bestCMax = cb.cMax;
            for(int i = 1;i<=n;i++)
            {
                bestC[i] = cb.c[i];
            }
        }

        found = true;
        return;
    }
    if(toSkip[ind]==true)
    {
        bruteForce(ind+1);
        return;
    }

    vector <int> v = {0, 1};
    if(rand()%2==0) swap(v[0], v[1]);

    for(int invert: v)
    {
        if(check(ind, invert)==true)
        {
            //cout << ind << " -- " << invert << '\n';

            apply(ind, invert, +1);
            bruteForce(ind+1);
            apply(ind, invert, -1);
        }
    }
}

void solveIndepentent()
{
    bestCMax = 0;
    for(int i = 0;i<m;i++)
    {
        int u = route2Nodes[i][0];
        int v = route2Nodes[i].back();

        if(bestC[u]>bestC[v])
        {
            reverse(route2Nodes[i].begin(), route2Nodes[i].end());
        }

        int ind = 1;
        for(int x: route2Nodes[i])
        {
            bestC[x] = max(bestC[x], ind);
            ind++;

            bestCMax = max(bestCMax, bestC[x]);
        }
    }
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> m >> T;
    for(int i = 0;i<n-1;i++)
    {
        int u, v;
        cin >> u >> v;

        edges.push_back(Edge(u, v, edges.size()));
        graph[u].push_back(edges.back());

        //cout << edges.back().u << " -> " << edges.back().v << " || " << edges.back().ind << '\n';

        edges.push_back(Edge(v, u, edges.size()));
        graph[v].push_back(edges.back());

        //cout << edges.back().u << " -> " << edges.back().v << " || " << edges.back().ind << '\n';
    }
    for(int i = 0;i<m;i++)
    {
        cin >> routes[i].first >> routes[i].second;
    }

    startTime = clock();

    Init();
    if(m<=15 || T==0)
    {
        bruteForce(0);
    }
    else
    {
        solveIndepentent();
    }

    if(bestCMax==MAXN)
    {
        cout << "-1" << '\n';
        return 0;
    }

    cout << bestCMax << '\n';

    cout << bestC[1];
    for(int i = 2;i<=n;i++)
    {
        cout << " " << bestC[i];
    }
    cout << '\n';

    return 0;
}
/*
5 3 0
1 2
1 3
1 4
1 5

1 2
1 3
4 2
*/
