#include <iostream>
#include <cstdint>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <algorithm>
#include <string>
#include <vector>
#include <cstring>

struct Path {
    int32_t begin;
    int32_t end;
    int32_t lca;
    int32_t length;

    Path(int32_t begin_, int32_t end_, int32_t lca_, int32_t length_) {
        begin = begin_;
        end = end_;
        lca = lca_;
        length = length_;
    }
};

void dfs(int32_t vertex, std::vector<int32_t>* tree, bool* visited,
         int32_t parent, int32_t* parents) {

    visited[vertex] = true;
    parents[vertex] = parent;
    for(int32_t i = 0; i < tree[vertex].size(); i++)
        if(!visited[tree[vertex][i]]) {
            dfs(tree[vertex][i], tree, visited,  vertex, parents);
        }
}

int32_t optimal_answer = INT32_MAX;
int32_t optimal_values[8];

void enumerate(int32_t step, int32_t target, std::vector<int32_t>& choices, bool* selected, std::vector<Path>& paths, int32_t* parents) {
    if(step == target) {
        for(int32_t i = 0 ; i < (1 << target); i++) {
            //enumerated everything
            int32_t* answer = new int32_t[8];
            for(int32_t j = 0 ; j < 8; j++)
                answer[j] = 0;

            bool ok = true;
            for(int32_t j = 0; j < target; j++) {
                //path j, direction in i
                int32_t begin = paths[choices[j]].begin;
                int32_t end = paths[choices[j]].end;
                if(i & (1 << choices[j])) {
                    int32_t tmp = begin;
                    begin = end;
                    end = tmp;
                }

                //assign
                int32_t cur = begin;
                int32_t step2 = std::max(answer[begin], 1);
                while(cur != parents[paths[choices[j]].lca]) {
                    if(answer[cur] != 0 && answer[cur] != step2) {
                        ok = false;
                        break;
                    }
                    answer[cur] = step2;
                    step2++;
                    cur = parents[cur];
                }
                if(!ok)
                    break;
                cur = end;
                step2 = answer[begin] + paths[choices[j]].length;
                while(cur != parents[paths[choices[j]].lca]) {
                    if(answer[cur] != 0 && answer[cur] != step2) {
                        ok = false;
                        break;
                    }
                    answer[cur] = step2;
                    step2--;
                    cur = parents[cur];
                }
                if(!ok)
                    break;
            }
            int32_t max_answer = answer[0];
            for(int32_t k = 0 ; k < 8; k++) {
                max_answer = std::max(max_answer, answer[k]);
                if(answer[k] < 0)
                    max_answer = INT32_MAX;
            }

            if(!ok) {
                delete[] answer;
                continue;
            }
            if(max_answer < optimal_answer) {
                optimal_answer = max_answer;
                memcpy(optimal_values, answer, sizeof(int32_t) * 8);
            }

            delete[] answer;
        }
    }
    for(int32_t i = 0 ; i < target; i++) {
        if(!selected[i]) {
            selected[i] = true;
            choices.push_back(i);
            enumerate(step + 1, target, choices, selected, paths, parents);
            choices.resize(step);
            selected[i] = false;
        }
    }
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(0);
    std::cout.tie(0);

    int32_t n, num_paths, t;
    std::cin >> n >> num_paths >> t;

    std::vector<int32_t>* tree = new std::vector<int32_t>[n];
    for(int32_t i = 0 ;i < n - 1; i++) {
        int32_t  src, dst;
        std::cin >> src >> dst;
        src--;
        dst--;
        tree[src].push_back(dst);
        tree[dst].push_back(src);
    }

    std::pair<int32_t, int32_t>* paths = new std::pair<int32_t, int32_t>[num_paths];
    for(int32_t i = 0 ; i < num_paths; i++) {
        std::cin >> paths[i].first >> paths[i].second;
        paths[i].first--;
        paths[i].second--;
    }

    bool* visited = new bool[n];
    for(int32_t i = 0 ;i < n; i++)
        visited[i] = false;
    int32_t* parents = new int32_t[n];

    dfs(0, tree, visited, -1, parents);

    std::vector<Path> paths2;
    for(int32_t i = 0 ; i < num_paths; i++) {
        std::vector<int32_t> path1;
        int32_t cur = paths[i].first;
        while(cur != -1) {
            path1.push_back(cur);
            cur = parents[cur];
        }
        std::vector<int32_t> path2;
        cur = paths[i].second;
        while(cur != -1) {
            path2.push_back(cur);
            cur = parents[cur];
        }
        std::reverse(path1.begin(), path1.end());
        std::reverse(path2.begin(), path2.end());
        int32_t lca = path1.size() < path2.size() ? paths[i].first : paths[i].second;
        int32_t distance = path1.size() + path2.size();
        for(int32_t k = 0; k < std::min(path1.size(), path2.size()); k++) {
            if (path1[k] != path2[k]) {
                lca = path1[k - 1];
                break;
            }
            distance -= 2;
        }
        paths2.emplace_back(paths[i].first, paths[i].second, lca, distance);
    }

    //solve
    std::vector<int32_t> choices;
    bool* selected = new bool[8];
    for(int32_t i = 0 ; i < n; i++)
        selected[i] = false;
    enumerate(0, num_paths, choices, selected, paths2, parents);

    if(optimal_answer == INT32_MAX)
        std::cout << -1;
    else {
        std::cout << optimal_answer << "\n";
        for (int32_t i = 0; i < n; i++)
            std::cout << std::max(1, optimal_values[i]) << " ";
    }
    return 0;
}